# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from tests.utils import assert_equal_dict

from polyaxon_schemas.optimizers import (
    AdadeltaConfig,
    AdagradConfig,
    AdamConfig,
    FtrlConfig,
    MomentumConfig,
    NestrovConfig,
    RMSPropConfig,
    SGDConfig
)


class TestOptimizerConfigs(TestCase):
    def test_sgd_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = SGDConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_momentum_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'momentum': 0.8,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = MomentumConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_nestrov_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'momentum': 0.8,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = NestrovConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_rmsprop_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'momentum': 0.,
            'decay': 0.9,
            'epsilon': 1e-10,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = RMSPropConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_adam_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'beta1': 0.9,
            'beta2': 0.999,
            'epsilon': 1e-10,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = AdamConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_adagrad_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'initial_accumulator_value': 0.1,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = AdagradConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_adadelta_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'epsilon': 1e-08,
            'rho': 0.95,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = AdadeltaConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_ftrl_config(self):
        config_dict = {
            'learning_rate': 0.001,
            'initial_accumulator_value': 0.1,
            'l1_regularization_strength': 0.0,
            'l2_regularization_strength': 0.0,
            'learning_rate_power': -0.5,
            'decay_type': "",
            'decay_rate': 0.,
            'decay_steps': 100,
            'start_decay_at': 0,
            'stop_decay_at': 1e10,
            'min_learning_rate': 1e-12,
            'staircase': False,
            'global_step': None,
            'use_locking': False,
            'name': 'optimizer'
        }
        config = FtrlConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import Schema, fields, post_dump, post_load

from polyaxon_schemas.base import BaseConfig


class BaseVersionSchema(Schema):
    latest_version = fields.Str()
    min_version = fields.Str()


class BaseVersionConfig(BaseConfig):
    def __init__(self, latest_version, min_version):
        self.latest_version = latest_version
        self.min_version = min_version


class CliVersionSchema(BaseVersionSchema):
    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return CliVersionConfig(**data)

    @post_dump
    def unmake(self, data):
        return CliVersionConfig.remove_reduced_attrs(data)


class CliVersionConfig(BaseVersionConfig):
    """
    Args:
        latest_version: the newest cli available on PIP.
        min_version: the version below which cli should fail.
    """
    SCHEMA = CliVersionSchema
    IDENTIFIER = 'cli_version'


class PlatformVersionSchema(BaseVersionSchema):
    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return PlatformVersionConfig(**data)

    @post_dump
    def unmake(self, data):
        return PlatformVersionConfig.remove_reduced_attrs(data)


class PlatformVersionConfig(BaseVersionConfig):
    """
    Args:
        latest_version: the newest platform available on helm.
        min_version: the version below which platform should fail.
    """
    SCHEMA = PlatformVersionSchema
    IDENTIFIER = 'platform_version'


class LibVersionSchema(BaseVersionSchema):
    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return LibVersionConfig(**data)

    @post_dump
    def unmake(self, data):
        return LibVersionConfig.remove_reduced_attrs(data)


class LibVersionConfig(BaseVersionConfig):
    """
    Args:
        latest_version: the newest lib available on PIP.
        min_version: the version below which lib should fail.
    """
    SCHEMA = LibVersionSchema
    IDENTIFIER = 'lib_version'


class ChartVersionSchema(Schema):
    version = fields.Str()

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ChartVersionConfig(**data)

    @post_dump
    def unmake(self, data):
        return ChartVersionConfig.remove_reduced_attrs(data)


class ChartVersionConfig(BaseConfig):
    """
    Args:
        version: the current installed chart version.
    """
    SCHEMA = ChartVersionSchema
    IDENTIFIER = 'chart_version'

    def __init__(self, version):
        self.version = version



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import fields, post_dump, post_load

from polyaxon_schemas.constraints import ConstraintSchema
from polyaxon_schemas.initializations import (
    InitializerSchema,
    OnesInitializerConfig,
    ZerosInitializerConfig
)
from polyaxon_schemas.layers.base import BaseLayerConfig, BaseLayerSchema
from polyaxon_schemas.regularizations import RegularizerSchema


class BatchNormalizationSchema(BaseLayerSchema):
    axis = fields.Int(default=-1, missing=-1)
    momentum = fields.Float(default=0.99, missing=0.99)
    epsilon = fields.Float(default=1e-3, missing=1e-3)
    center = fields.Bool(default=True, missing=True)
    scale = fields.Bool(default=True, missing=True)
    beta_initializer = fields.Nested(InitializerSchema, allow_none=True)
    gamma_initializer = fields.Nested(InitializerSchema, allow_none=True)
    moving_mean_initializer = fields.Nested(InitializerSchema, allow_none=True)
    moving_variance_initializer = fields.Nested(InitializerSchema, allow_none=True)
    beta_regularizer = fields.Nested(RegularizerSchema, allow_none=True)
    gamma_regularizer = fields.Nested(RegularizerSchema, allow_none=True)
    beta_constraint = fields.Nested(ConstraintSchema, allow_none=True)
    gamma_constraint = fields.Nested(ConstraintSchema, allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return BatchNormalizationConfig(**data)

    @post_dump
    def unmake(self, data):
        return BatchNormalizationConfig.remove_reduced_attrs(data)


class BatchNormalizationConfig(BaseLayerConfig):
    """Batch normalization layer (Ioffe and Szegedy, 2014).

    Normalize the activations of the previous layer at each batch,
    i.e. applies a transformation that maintains the mean activation
    close to 0 and the activation standard deviation close to 1.

    Args:
        axis: Integer, the axis that should be normalized
            (typically the features axis).
            For instance, after a `Conv2D` layer with
            `data_format="channels_first"`,
            set `axis=1` in `BatchNormalization`.
        momentum: Momentum for the moving average.
        epsilon: Small float added to variance to avoid dividing by zero.
        center: If True, add offset of `beta` to normalized tensor.
            If False, `beta` is ignored.
        scale: If True, multiply by `gamma`.
            If False, `gamma` is not used.
            When the next layer is linear (also e.g. `nn.relu`),
            this can be disabled since the scaling
            will be done by the next layer.
        beta_initializer: Initializer for the beta weight.
        gamma_initializer: Initializer for the gamma weight.
        moving_mean_initializer: Initializer for the moving mean.
        moving_variance_initializer: Initializer for the moving variance.
        beta_regularizer: Optional regularizer for the beta weight.
        gamma_regularizer: Optional regularizer for the gamma weight.
        beta_constraint: Optional constraint for the beta weight.
        gamma_constraint: Optional constraint for the gamma weight.

    Input shape:
        Arbitrary. Use the keyword argument `input_shape`
        (tuple of integers, does not include the samples axis)
        when using this layer as the first layer in a model.

    Output shape:
        Same shape as input.

    References:
        - [Batch Normalization: Accelerating Deep Network Training by Reducing
          Internal Covariate Shift](https://arxiv.org/abs/1502.03167)

    Polyaxonfile usage:

    ```yaml
    BatchNormalization:
      momentum: 0.7
    ```
    """
    IDENTIFIER = 'BatchNormalization'
    SCHEMA = BatchNormalizationSchema

    def __init__(self,
                 axis=-1,
                 momentum=0.99,
                 epsilon=1e-3,
                 center=True,
                 scale=True,
                 beta_initializer=ZerosInitializerConfig(),
                 gamma_initializer=OnesInitializerConfig(),
                 moving_mean_initializer=ZerosInitializerConfig(),
                 moving_variance_initializer=OnesInitializerConfig(),
                 beta_regularizer=None,
                 gamma_regularizer=None,
                 beta_constraint=None,
                 gamma_constraint=None,
                 **kwargs):
        super(BatchNormalizationConfig, self).__init__(**kwargs)
        self.axis = axis
        self.momentum = momentum
        self.epsilon = epsilon
        self.center = center
        self.scale = scale
        self.beta_initializer = beta_initializer
        self.gamma_initializer = gamma_initializer
        self.moving_mean_initializer = moving_mean_initializer
        self.moving_variance_initializer = moving_variance_initializer
        self.beta_regularizer = beta_regularizer
        self.gamma_regularizer = gamma_regularizer
        self.beta_constraint = beta_constraint
        self.gamma_constraint = gamma_constraint



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os

from unittest import TestCase

from polyaxon_schemas.exceptions import PolyaxonConfigurationError, PolyaxonfileError
from polyaxon_schemas.polyaxonfile.specification import (
    BuildSpecification,
    ExperimentSpecification,
    JobSpecification,
    NotebookSpecification,
    TensorboardSpecification
)
from polyaxon_schemas.utils import TaskType


class TestSpecifications(TestCase):
    def test_notebook_specification_raises_for_invalid_run_section(self):
        with self.assertRaises(PolyaxonfileError):
            NotebookSpecification.read({'version': 1, 'kind': 'notebook'})

        with self.assertRaises(PolyaxonfileError):
            NotebookSpecification.read(os.path.abspath(
                'tests/fixtures/notebook_run_exec_simple_file_with_cmd.yml'))

    def test_tensorboard_specification_raises_for_invalid_run_section(self):
        with self.assertRaises(PolyaxonfileError):
            TensorboardSpecification.read({'version': 1, 'kind': 'tensorboard'})

        with self.assertRaises(PolyaxonConfigurationError):
            TensorboardSpecification.read(os.path.abspath(
                'tests/fixtures/tensorboard_run_exec_simple_file_with_cmd.yml'))

    def test_job_specification_raises_for_missing_build_section(self):
        with self.assertRaises(PolyaxonfileError):
            JobSpecification.read(os.path.abspath(
                'tests/fixtures/job_missing_build.yml'))

    def test_job_specification_raises_for_missing_run_section(self):
        with self.assertRaises(PolyaxonfileError):
            JobSpecification.read(os.path.abspath(
                'tests/fixtures/job_missing_run_exec.yml'))

    def test_create_notebook_specification(self):
        build_config = {'image': 'blabla'}
        config = NotebookSpecification.create_specification(build_config)
        assert NotebookSpecification.read(config).parsed_data == config
        assert config['build'] == build_config
        spec = NotebookSpecification.create_specification(build_config, to_dict=False)
        assert spec.build.image == build_config['image']

    def test_create_tensorboard_specification(self):
        build_config = {'image': 'blabla'}
        config = TensorboardSpecification.create_specification(build_config)
        assert TensorboardSpecification.read(config).parsed_data == config
        assert config['build'] == build_config
        spec = TensorboardSpecification.create_specification(build_config, to_dict=False)
        assert spec.build.image == build_config['image']

    def test_create_build_specification(self):
        # Normal build config
        build_config = {'image': 'blabla'}
        config = BuildSpecification.create_specification(build_config)
        assert BuildSpecification.read(config).parsed_data == config
        assert config['build'] == build_config
        spec = BuildSpecification.create_specification(build_config, to_dict=False)
        assert spec.build.image == build_config['image']

        # Run config
        run_config = {'image': 'blabla', 'cmd': 'some command'}
        config = BuildSpecification.create_specification(run_config)
        assert BuildSpecification.read(config).parsed_data == config
        assert config['build'] == {'image': 'blabla'}
        spec = BuildSpecification.create_specification(run_config, to_dict=False)
        assert spec.build.image == run_config['image']

    def test_create_job_specification(self):
        build_config = {'image': 'blabla'}
        run_config = {'cmd': 'some command'}
        config = JobSpecification.create_specification(build_config=build_config,
                                                       run_config=run_config)
        assert JobSpecification.read(config).parsed_data == config
        assert config['run'] == run_config
        assert config['build'] == build_config
        spec = JobSpecification.create_specification(build_config=build_config,
                                                     run_config=run_config,
                                                     to_dict=False)
        assert spec.build.image == build_config['image']
        assert spec.run.cmd == run_config['cmd']

    def test_cluster_def_without_framework(self):
        spec = ExperimentSpecification.read(os.path.abspath(
            'tests/fixtures/env_without_framework.yml'))
        self.assertEqual(spec.cluster_def, ({TaskType.MASTER: 1}, False))

    def test_patch(self):
        content = {
            'version': 1,
            'kind': 'experiment',
            'build': {'image': 'my_image'},
            'run': {'cmd': 'train'}
        }
        spec = ExperimentSpecification.read(content)
        assert spec.declarations is None

        # Add declarations
        declarations = {'declarations': {'lr': 0.1}}
        spec = spec.patch(values=declarations)
        assert spec.declarations == declarations['declarations']

        # Update declarations
        declarations = {'declarations': {'lr': 0.01, 'num_steps': 100}}
        spec = spec.patch(values=declarations)
        assert spec.declarations == declarations['declarations']

        # Add env
        assert spec.environment is None
        env = {'environment': {'resources': {'gpu': {'requests': 1, 'limits': 1}}}}
        spec = spec.patch(values=env)
        assert spec.declarations == declarations['declarations']
        assert spec.environment.resources.gpu.to_dict() == env['environment']['resources']['gpu']

        # Patch with unsupported spec
        matrix = {'hptuning': {'matrix': {'lr': {'values': [0.1, 0.2]}}}}
        with self.assertRaises(PolyaxonfileError):
            spec.patch(values=matrix)

        # Patch with unsupported spec
        wrong_config = {'lr': {'values': [0.1, 0.2]}}
        with self.assertRaises(PolyaxonfileError):
            spec.patch(values=wrong_config)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from marshmallow import ValidationError
from tests.utils import assert_equal_dict

from polyaxon_schemas.environments import (
    EnvironmentConfig,
    GPUOptionsConfig,
    HorovodClusterConfig,
    HorovodConfig,
    K8SResourcesConfig,
    MXNetClusterConfig,
    MXNetConfig,
    PodResourcesConfig,
    PytorchClusterConfig,
    PytorchConfig,
    SessionConfig,
    TensorflowClusterConfig,
    TensorflowConfig,
    TFRunConfig
)
from polyaxon_schemas.utils import TaskType


class TestEnvironmentsConfigs(TestCase):
    def test_k8s_resources_config(self):
        config_dict = {
            'requests': 0.8,
            'limits': 1,
        }
        config = K8SResourcesConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_pod_resources_config(self):
        config_dict = {
            'cpu': {
                'requests': 0.8,
                'limits': 1
            },
            'gpu': {
                'requests': 2,
                'limits': 4
            },
            'memory': {
                'requests': 265,
                'limits': 512
            },
        }
        config = PodResourcesConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_pod_resources_add(self):
        config_dict1 = {
            'cpu': {
                'requests': 0.8,
            },
            'gpu': {
                'requests': 2,
            },
            'memory': {
                'requests': 200,
                'limits': 300
            },
        }

        config_dict2 = {
            'gpu': {
                'limits': 4
            },
            'memory': {
                'requests': 300,
                'limits': 200
            },
        }
        config1 = PodResourcesConfig.from_dict(config_dict1)
        config2 = PodResourcesConfig.from_dict(config_dict2)

        config = config1 + config2
        assert config.cpu.to_dict() == {'requests': 0.8, 'limits': None}
        assert config.memory.to_dict() == {'requests': 500, 'limits': 500}
        assert config.gpu.to_dict() == {'requests': 2, 'limits': 4}

    def test_gpu_options_config(self):
        config_dict = {
            'gpu_memory_fraction': 0.8,
            'allow_growth': False,
            'per_process_gpu_memory_fraction': 0.4,
        }
        config = GPUOptionsConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_session_config(self):
        config_dict = {
            'log_device_placement': False,
            'allow_soft_placement': False,
            'intra_op_parallelism_threads': 2,
            'inter_op_parallelism_threads': 3,
            'gpu_options': GPUOptionsConfig().to_dict(),
        }
        config = SessionConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_indexed_session(self):
        config_dict = {
            'index': 10,
            'log_device_placement': False,
            'allow_soft_placement': False,
            'intra_op_parallelism_threads': 2,
            'inter_op_parallelism_threads': 3,
            'gpu_options': GPUOptionsConfig().to_dict(),
        }
        config = SessionConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_tensorflow_cluster_config(self):
        config_dict = {
            "worker": [
                "worker0.example.com:2222",
                "worker1.example.com:2222",
                "worker2.example.com:2222"
            ],
            "ps": [
                "ps0.example.com:2222",
                "ps1.example.com:2222"
            ]
        }
        config = TensorflowClusterConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_horovod_cluster_config(self):
        config_dict = {
            "worker": [
                "worker0.example.com:2222",
                "worker1.example.com:2222",
                "worker2.example.com:2222"
            ]
        }
        config = HorovodClusterConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_pytorch_cluster_config(self):
        config_dict = {
            "worker": [
                "worker0.example.com:2222",
                "worker1.example.com:2222",
                "worker2.example.com:2222"
            ]
        }
        config = PytorchClusterConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_mxnet_cluster_config(self):
        config_dict = {
            "worker": [
                "worker0.example.com:2222",
                "worker1.example.com:2222",
                "worker2.example.com:2222"
            ],
            "server": [
                "server0.example.com:2222",
                "server1.example.com:2222"
            ]
        }
        config = MXNetClusterConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_run_config(self):
        config_dict = {
            'tf_random_seed': 100,
            'save_summary_steps': 100,
            'save_checkpoints_secs': 600,
            'save_checkpoints_steps': None,
            'keep_checkpoint_max': 5,
            'keep_checkpoint_every_n_hours': 10000,
        }
        config = TFRunConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

        # Add session config
        config_dict['session'] = SessionConfig().to_dict()
        config = TFRunConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

        # Add cluster config
        config_dict['cluster'] = TensorflowClusterConfig(
            worker=[TaskType.WORKER], ps=[TaskType.PS]
        ).to_dict()
        config = TFRunConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_tensorflow_config(self):
        config_dict = {
            'n_workers': 10,
            'n_ps': 5,
            'delay_workers_by_global_step': False
        }
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add run config
        config_dict['run_config'] = TFRunConfig().to_dict()
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default worker session config
        config_dict['default_worker_config'] = SessionConfig(
            intra_op_parallelism_threads=1,
            inter_op_parallelism_threads=3
        ).to_dict()
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default worker resources
        config_dict['default_worker_resources'] = PodResourcesConfig(
            cpu=K8SResourcesConfig(0.5, 1), gpu=K8SResourcesConfig(2, 4)).to_dict()
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default ps session config
        config_dict['default_ps_config'] = SessionConfig(
            intra_op_parallelism_threads=0,
            inter_op_parallelism_threads=2
        ).to_dict()
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default ps resources
        config_dict['default_ps_resources'] = PodResourcesConfig(
            cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom config for worker 3
        config_dict['worker_configs'] = [SessionConfig(
            index=3,
            gpu_options=GPUOptionsConfig(gpu_memory_fraction=0.4),
            intra_op_parallelism_threads=8,
            inter_op_parallelism_threads=8
        ).to_dict()]
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom resources for worker 4
        config_dict['worker_resources'] = [PodResourcesConfig(
            index=4, cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()]
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom config for ps 2
        config_dict['ps_configs'] = [SessionConfig(
            index=2,
            gpu_options=GPUOptionsConfig(allow_growth=False),
            intra_op_parallelism_threads=1,
            inter_op_parallelism_threads=1
        ).to_dict()]
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom resources for ps 4
        config_dict['ps_resources'] = [PodResourcesConfig(
            index=4, cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()]
        config = TensorflowConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_horovod_config(self):
        config_dict = {
            'n_workers': 10,
        }
        config = HorovodConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default worker resources
        config_dict['default_worker_resources'] = PodResourcesConfig(
            cpu=K8SResourcesConfig(0.5, 1), gpu=K8SResourcesConfig(2, 4)).to_dict()
        config = HorovodConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom resources for worker 4
        config_dict['worker_resources'] = [PodResourcesConfig(
            index=4, cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()]
        config = HorovodConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_pytorch_config(self):
        config_dict = {
            'n_workers': 10,
        }
        config = PytorchConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default worker resources
        config_dict['default_worker_resources'] = PodResourcesConfig(
            cpu=K8SResourcesConfig(0.5, 1), gpu=K8SResourcesConfig(2, 4)).to_dict()
        config = PytorchConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom resources for worker 4
        config_dict['worker_resources'] = [PodResourcesConfig(
            index=4, cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()]
        config = PytorchConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_mxnet_config(self):
        config_dict = {
            'n_workers': 10,
            'n_ps': 5,
        }
        config = MXNetConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default worker resources
        config_dict['default_worker_resources'] = PodResourcesConfig(
            cpu=K8SResourcesConfig(0.5, 1), gpu=K8SResourcesConfig(2, 4)).to_dict()
        config = MXNetConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add default ps resources
        config_dict['default_ps_resources'] = PodResourcesConfig(
            cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()
        config = MXNetConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom resources for worker 4
        config_dict['worker_resources'] = [PodResourcesConfig(
            index=4, cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()]
        config = MXNetConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding custom resources for ps 4
        config_dict['ps_resources'] = [PodResourcesConfig(
            index=4, cpu=K8SResourcesConfig(0.5, 1), memory=K8SResourcesConfig(256, 400)).to_dict()]
        config = MXNetConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

    def test_environment_config(self):
        config_dict = {
            'resources': PodResourcesConfig(cpu=K8SResourcesConfig(0.5, 1)).to_dict()
        }
        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add node selectors
        config_dict['node_selectors'] = {
            'polyaxon.com': 'master',
        }

        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add persistence
        config_dict['persistence'] = {
            'data': ['data1', 'data2'],
            'outputs': 'outputs1',
        }
        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add outputs
        config_dict['outputs'] = {
            'jobs': ['data1.dfs', 34, 'data2'],
            'experiments': [1, 'outputs1', 2, 3],
        }
        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add tensorflow
        config_dict['tensorflow'] = {
            'n_workers': 10,
            'n_ps': 5,
        }

        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add mxnet should raise
        config_dict['mxnet'] = {
            'n_workers': 10,
            'n_ps': 5,
        }

        with self.assertRaises(ValidationError):
            EnvironmentConfig.from_dict(config_dict)

        # Removing tensorflow should pass for mxnet
        del config_dict['tensorflow']
        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding horovod should raise
        config_dict['horovod'] = {
            'n_workers': 5
        }

        with self.assertRaises(ValidationError):
            EnvironmentConfig.from_dict(config_dict)

        # Removing mxnet should pass for horovod
        del config_dict['mxnet']
        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Adding pytorch should raise
        config_dict['pytorch'] = {
            'n_workers': 5
        }

        with self.assertRaises(ValidationError):
            EnvironmentConfig.from_dict(config_dict)

        # Removing horovod should pass for pytorch
        del config_dict['horovod']
        config = EnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import Schema, fields, post_dump, post_load

from polyaxon_schemas.base import BaseConfig


class UserSchema(Schema):
    username = fields.Str()
    email = fields.Email(allow_none=True)
    is_superuser = fields.Bool(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return UserConfig(**data)

    @post_dump
    def unmake(self, data):
        return UserConfig.remove_reduced_attrs(data)


class UserConfig(BaseConfig):
    SCHEMA = UserSchema
    IDENTIFIER = 'user'

    def __init__(self, username, email, is_superuser=False):
        self.username = username
        self.email = email
        self.is_superuser = is_superuser



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import Schema, ValidationError, fields, post_dump, post_load, validates_schema

from polyaxon_schemas.base import BaseConfig
from polyaxon_schemas.utils import UUID, IntOrStr


class TensorflowClusterSchema(Schema):
    master = fields.List(fields.Str(), allow_none=True)
    worker = fields.List(fields.Str(), allow_none=True)
    ps = fields.List(fields.Str(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return TensorflowClusterConfig(**data)

    @post_dump
    def unmake(self, data):
        return TensorflowClusterConfig.remove_reduced_attrs(data)


class TensorflowClusterConfig(BaseConfig):
    """Tensorflow cluster definition"""
    IDENTIFIER = 'tensorflow_cluster'
    SCHEMA = TensorflowClusterSchema

    def __init__(self, master=None, worker=None, ps=None):
        self.master = master
        self.worker = worker
        self.ps = ps


class HorovodClusterSchema(Schema):
    master = fields.List(fields.Str(), allow_none=True)
    worker = fields.List(fields.Str(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return HorovodClusterConfig(**data)

    @post_dump
    def unmake(self, data):
        return HorovodClusterConfig.remove_reduced_attrs(data)


class HorovodClusterConfig(BaseConfig):
    """Horovod cluster definition"""
    IDENTIFIER = 'horovod_cluster'
    SCHEMA = HorovodClusterSchema

    def __init__(self, master=None, worker=None):
        self.master = master
        self.worker = worker


class PytorchClusterSchema(Schema):
    master = fields.List(fields.Str(), allow_none=True)
    worker = fields.List(fields.Str(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return PytorchClusterConfig(**data)

    @post_dump
    def unmake(self, data):
        return PytorchClusterConfig.remove_reduced_attrs(data)


class PytorchClusterConfig(BaseConfig):
    """Pytorch cluster definition"""
    IDENTIFIER = 'pytorch_cluster'
    SCHEMA = PytorchClusterSchema

    def __init__(self, master=None, worker=None):
        self.master = master
        self.worker = worker


class MXNetClusterSchema(Schema):
    master = fields.List(fields.Str(), allow_none=True)
    worker = fields.List(fields.Str(), allow_none=True)
    server = fields.List(fields.Str(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return MXNetClusterConfig(**data)

    @post_dump
    def unmake(self, data):
        return MXNetClusterConfig.remove_reduced_attrs(data)


class MXNetClusterConfig(BaseConfig):
    """MXNet cluster definition"""
    IDENTIFIER = 'mxnet_cluster'
    SCHEMA = MXNetClusterSchema

    def __init__(self, master=None, worker=None, server=None):
        self.master = master
        self.worker = worker
        self.server = server


class K8SResourcesSchema(Schema):
    limits = fields.Float(allow_none=True)
    requests = fields.Float(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return K8SResourcesConfig(**data)

    @post_dump
    def unmake(self, data):
        return K8SResourcesConfig.remove_reduced_attrs(data)


class K8SResourcesConfig(BaseConfig):
    IDENTIFIER = 'resources'
    SCHEMA = K8SResourcesSchema

    def __init__(self, limits=None, requests=None):
        self.limits = limits
        self.requests = requests

    def __add__(self, other):
        if not other:
            return self

        if self.requests:
            if other.requests:
                self.requests += other.requests
        elif other.requests:
            self.requests = other.requests

        if self.limits:
            if other.limits:
                self.limits += other.limits
        elif other.limits:
            self.limits = other.limits

        return self


class PodResourcesSchema(Schema):
    # To indicate which worker/ps index this session config belongs to
    index = fields.Int(allow_none=True)
    cpu = fields.Nested(K8SResourcesSchema, allow_none=True)
    memory = fields.Nested(K8SResourcesSchema, allow_none=True)
    gpu = fields.Nested(K8SResourcesSchema, allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return PodResourcesConfig(**data)

    @post_dump
    def unmake(self, data):
        return PodResourcesConfig.remove_reduced_attrs(data)


class PodResourcesConfig(BaseConfig):
    IDENTIFIER = 'pod_resources'
    SCHEMA = PodResourcesSchema
    REDUCED_ATTRIBUTES = ['index']

    def __init__(self, index=None, cpu=None, memory=None, gpu=None):
        self.index = index
        self.cpu = cpu
        self.memory = memory
        self.gpu = gpu

    def __add__(self, other):
        if not other:
            return self

        if self.cpu:
            if other.cpu:
                self.cpu += other.cpu
        elif other.cpu:
            self.cpu = K8SResourcesConfig()
            self.cpu += other.cpu

        if self.memory:
            if other.memory:
                self.memory += other.memory
        elif other.memory:
            self.memory = K8SResourcesConfig()
            self.memory += other.memory

        if self.gpu:
            if other.gpu:
                self.gpu += other.gpu
        elif other.gpu:
            self.gpu = K8SResourcesConfig()
            self.gpu += other.gpu
        return self


class GPUOptionsSchema(Schema):
    gpu_memory_fraction = fields.Float(allow_none=True)
    allow_growth = fields.Bool(allow_none=True)
    per_process_gpu_memory_fraction = fields.Float(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return GPUOptionsConfig(**data)

    @post_dump
    def unmake(self, data):
        return GPUOptionsConfig.remove_reduced_attrs(data)


class GPUOptionsConfig(BaseConfig):
    IDENTIFIER = 'session'
    SCHEMA = GPUOptionsSchema

    def __init__(self,
                 gpu_memory_fraction=None,
                 allow_growth=True,
                 per_process_gpu_memory_fraction=None):
        self.gpu_memory_fraction = gpu_memory_fraction
        self.allow_growth = allow_growth
        self.per_process_gpu_memory_fraction = per_process_gpu_memory_fraction


class SessionSchema(Schema):
    # To indicate which worker/ps index this session config belongs to
    index = fields.Int(allow_none=True)
    gpu_options = fields.Nested(GPUOptionsSchema, allow_none=True)
    log_device_placement = fields.Bool(allow_none=True)
    allow_soft_placement = fields.Bool(allow_none=True)
    intra_op_parallelism_threads = fields.Int(allow_none=True)
    inter_op_parallelism_threads = fields.Int(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return SessionConfig(**data)

    @post_dump
    def unmake(self, data):
        return SessionConfig.remove_reduced_attrs(data)


class SessionConfig(BaseConfig):
    IDENTIFIER = 'session'
    SCHEMA = SessionSchema
    REDUCED_ATTRIBUTES = ['index']

    def __init__(self,
                 index=None,
                 log_device_placement=True,
                 gpu_options=GPUOptionsConfig(),
                 allow_soft_placement=True,
                 intra_op_parallelism_threads=None,
                 inter_op_parallelism_threads=None):
        self.index = index
        self.gpu_options = gpu_options
        self.log_device_placement = log_device_placement
        self.allow_soft_placement = allow_soft_placement
        self.intra_op_parallelism_threads = intra_op_parallelism_threads
        self.inter_op_parallelism_threads = inter_op_parallelism_threads


class TFRunSchema(Schema):
    tf_random_seed = fields.Int(allow_none=True)
    save_summary_steps = fields.Int(allow_none=True)
    save_checkpoints_secs = fields.Int(allow_none=True)
    save_checkpoints_steps = fields.Int(allow_none=True)
    keep_checkpoint_max = fields.Int(allow_none=True)
    keep_checkpoint_every_n_hours = fields.Int(allow_none=True)

    session = fields.Nested(SessionSchema, allow_none=True)
    cluster = fields.Nested(TensorflowClusterSchema, allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return TFRunConfig(**data)

    @post_dump
    def unmake(self, data):
        return TFRunConfig.remove_reduced_attrs(data)


class TFRunConfig(BaseConfig):
    IDENTIFIER = 'run'
    SCHEMA = TFRunSchema

    def __init__(self,
                 tf_random_seed=None,
                 save_summary_steps=100,
                 save_checkpoints_secs=None,
                 save_checkpoints_steps=None,
                 keep_checkpoint_max=5,
                 keep_checkpoint_every_n_hours=10000,
                 session=None,
                 cluster=None):
        self.tf_random_seed = tf_random_seed
        self.save_summary_steps = save_summary_steps
        self.save_checkpoints_secs = save_checkpoints_secs
        self.save_checkpoints_steps = save_checkpoints_steps
        self.keep_checkpoint_max = keep_checkpoint_max
        self.keep_checkpoint_every_n_hours = keep_checkpoint_every_n_hours
        self.session = session
        self.cluster = cluster


class TensorflowSchema(Schema):
    n_workers = fields.Int(allow_none=True)
    n_ps = fields.Int(allow_none=True)
    delay_workers_by_global_step = fields.Bool(allow_none=True)
    run_config = fields.Nested(TFRunSchema, allow_none=True)
    default_worker_config = fields.Nested(SessionSchema, allow_none=True)
    default_worker_resources = fields.Nested(PodResourcesSchema, allow_none=True)
    default_worker_node_selectors = fields.Dict(allow_none=True)
    default_ps_config = fields.Nested(SessionSchema, allow_none=True)
    default_ps_resources = fields.Nested(PodResourcesSchema, allow_none=True)
    default_ps_node_selectors = fields.Dict(allow_none=True)
    worker_configs = fields.Nested(SessionSchema, many=True, allow_none=True)
    worker_resources = fields.Nested(PodResourcesSchema, many=True, allow_none=True)
    ps_configs = fields.Nested(SessionSchema, many=True, allow_none=True)
    ps_resources = fields.Nested(PodResourcesSchema, many=True, allow_none=True)
    worker_node_selectors = fields.List(fields.Dict(), allow_none=True)
    ps_node_selectors = fields.List(fields.Dict(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return TensorflowConfig(**data)

    @post_dump
    def unmake(self, data):
        return TensorflowConfig.remove_reduced_attrs(data)


class TensorflowConfig(BaseConfig):
    IDENTIFIER = 'tensorflow'
    SCHEMA = TensorflowSchema

    def __init__(self,
                 n_workers=0,
                 n_ps=0,
                 delay_workers_by_global_step=False,
                 run_config=None,
                 default_worker_config=None,
                 default_worker_resources=None,
                 default_ps_config=None,
                 default_ps_resources=None,
                 default_worker_node_selectors=None,
                 default_ps_node_selectors=None,
                 worker_configs=None,
                 worker_resources=None,
                 ps_configs=None,
                 ps_resources=None,
                 worker_node_selectors=None,
                 ps_node_selectors=None):
        self.n_workers = n_workers
        self.n_ps = n_ps
        self.delay_workers_by_global_step = delay_workers_by_global_step
        self.run_config = run_config
        self.default_worker_config = default_worker_config
        self.default_worker_resources = default_worker_resources
        self.default_ps_config = default_ps_config
        self.default_ps_resources = default_ps_resources
        self.default_worker_node_selectors = default_worker_node_selectors
        self.default_ps_node_selectors = default_ps_node_selectors
        self.worker_configs = worker_configs
        self.worker_resources = worker_resources
        self.ps_configs = ps_configs
        self.ps_resources = ps_resources
        self.worker_node_selectors = worker_node_selectors
        self.ps_node_selectors = ps_node_selectors


class HorovodSchema(Schema):
    n_workers = fields.Int(allow_none=True)
    default_worker_resources = fields.Nested(PodResourcesSchema, allow_none=True)
    default_worker_node_selectors = fields.Dict(allow_none=True)
    worker_resources = fields.Nested(PodResourcesSchema, many=True, allow_none=True)
    worker_node_selectors = fields.List(fields.Dict(allow_none=True), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return HorovodConfig(**data)

    @post_dump
    def unmake(self, data):
        return HorovodConfig.remove_reduced_attrs(data)


class HorovodConfig(BaseConfig):
    IDENTIFIER = 'horovod'
    SCHEMA = HorovodSchema

    def __init__(self,
                 n_workers=0,
                 default_worker_resources=None,
                 default_worker_node_selectors=None,
                 worker_resources=None,
                 worker_node_selectors=None):
        self.n_workers = n_workers
        self.default_worker_resources = default_worker_resources
        self.default_worker_node_selectors = default_worker_node_selectors
        self.worker_resources = worker_resources
        self.worker_node_selectors = worker_node_selectors


class PytorchSchema(Schema):
    n_workers = fields.Int(allow_none=True)
    default_worker_resources = fields.Nested(PodResourcesSchema, allow_none=True)
    default_worker_node_selectors = fields.Dict(allow_none=True)
    worker_resources = fields.Nested(PodResourcesSchema, many=True, allow_none=True)
    worker_node_selectors = fields.List(fields.Dict(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return PytorchConfig(**data)

    @post_dump
    def unmake(self, data):
        return PytorchConfig.remove_reduced_attrs(data)


class PytorchConfig(BaseConfig):
    IDENTIFIER = 'pytorch'
    SCHEMA = PytorchSchema

    def __init__(self,
                 n_workers=0,
                 default_worker_resources=None,
                 default_worker_node_selectors=None,
                 worker_resources=None,
                 worker_node_selectors=None):
        self.n_workers = n_workers
        self.default_worker_resources = default_worker_resources
        self.default_worker_node_selectors = default_worker_node_selectors
        self.worker_resources = worker_resources
        self.worker_node_selectors = worker_node_selectors


class MXNetSchema(Schema):
    n_workers = fields.Int(allow_none=True)
    n_ps = fields.Int(allow_none=True)
    default_worker_resources = fields.Nested(PodResourcesSchema, allow_none=True)
    default_ps_resources = fields.Nested(PodResourcesSchema, allow_none=True)
    default_worker_node_selectors = fields.Dict(allow_none=True)
    default_ps_node_selectors = fields.Dict(allow_none=True)
    worker_resources = fields.Nested(PodResourcesSchema, many=True, allow_none=True)
    ps_resources = fields.Nested(PodResourcesSchema, many=True, allow_none=True)
    worker_node_selectors = fields.List(fields.Dict(), allow_none=True)
    ps_node_selectors = fields.List(fields.Dict(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return MXNetConfig(**data)

    @post_dump
    def unmake(self, data):
        return MXNetConfig.remove_reduced_attrs(data)


class MXNetConfig(BaseConfig):
    IDENTIFIER = 'mxnet'
    SCHEMA = MXNetSchema

    def __init__(self,
                 n_workers=0,
                 n_ps=0,
                 default_worker_resources=None,
                 default_ps_resources=None,
                 default_worker_node_selectors=None,
                 default_ps_node_selectors=None,
                 worker_resources=None,
                 ps_resources=None,
                 worker_node_selectors=None,
                 ps_node_selectors=None):
        self.n_workers = n_workers
        self.n_ps = n_ps
        self.default_worker_resources = default_worker_resources
        self.default_ps_resources = default_ps_resources
        self.default_worker_node_selectors = default_worker_node_selectors
        self.default_ps_node_selectors = default_ps_node_selectors
        self.worker_resources = worker_resources
        self.ps_resources = ps_resources
        self.worker_node_selectors = worker_node_selectors
        self.ps_node_selectors = ps_node_selectors


def validate_frameworks(frameworks):
    if sum([1 for f in frameworks if f is not None]) > 1:
        raise ValidationError('Only one framework can be used.')


class PersistenceSchema(Schema):
    data = fields.List(fields.Str(), allow_none=True)
    outputs = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return PersistenceConfig(**data)

    @post_dump
    def unmake(self, data):
        return PersistenceConfig.remove_reduced_attrs(data)


class PersistenceConfig(BaseConfig):
    IDENTIFIER = 'persistence'
    SCHEMA = PersistenceSchema

    def __init__(self, data=None, outputs=None):
        self.data = data
        self.outputs = outputs


class OutputsSchema(Schema):
    jobs = fields.List(IntOrStr(), allow_none=True)
    experiments = fields.List(IntOrStr(), allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return OutputsConfig(**data)

    @post_dump
    def unmake(self, data):
        return OutputsConfig.remove_reduced_attrs(data)


class OutputsConfig(BaseConfig):
    IDENTIFIER = 'outputs'
    SCHEMA = OutputsSchema

    def __init__(self, jobs=None, experiments=None):
        self.jobs = jobs
        self.experiments = experiments


class EnvironmentSchema(Schema):
    cluster_uuid = UUID(allow_none=True)
    resources = fields.Nested(PodResourcesSchema, allow_none=True)
    persistence = fields.Nested(PersistenceSchema, allow_none=True)
    outputs = fields.Nested(OutputsSchema, allow_none=True)
    node_selectors = fields.Dict(allow_none=True)
    tensorflow = fields.Nested(TensorflowSchema, allow_none=True)
    horovod = fields.Nested(HorovodSchema, allow_none=True)
    mxnet = fields.Nested(MXNetSchema, allow_none=True)
    pytorch = fields.Nested(PytorchSchema, allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return EnvironmentConfig(**data)

    @post_dump
    def unmake(self, data):
        return EnvironmentConfig.remove_reduced_attrs(data)

    @validates_schema
    def validate_quantity(self, data):
        validate_frameworks([data.get('tensorflow'),
                             data.get('mxnet'),
                             data.get('pytorch'),
                             data.get('horovod')])


class EnvironmentConfig(BaseConfig):
    IDENTIFIER = 'environment'
    SCHEMA = EnvironmentSchema

    def __init__(self,
                 cluster_uuid=None,
                 resources=None,
                 persistence=None,
                 outputs=None,
                 node_selectors=None,
                 tensorflow=None,
                 horovod=None,
                 pytorch=None,
                 mxnet=None):
        self.cluster_uuid = cluster_uuid
        self.resources = resources
        self.persistence = persistence
        self.outputs = outputs
        self.node_selectors = node_selectors
        validate_frameworks([tensorflow, horovod, pytorch, mxnet])
        self.tensorflow = tensorflow
        self.horovod = horovod
        self.pytorch = pytorch
        self.mxnet = mxnet



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_schemas.polyaxonfile.specification.base import BaseSpecification
from polyaxon_schemas.polyaxonfile.specification.frameworks import (
    HorovodSpecification,
    MXNetSpecification,
    PytorchSpecification,
    TensorflowSpecification
)
from polyaxon_schemas.polyaxonfile.specification.job import JobSpecification
from polyaxon_schemas.polyaxonfile.utils import cached_property
from polyaxon_schemas.utils import Frameworks, TaskType


class ExperimentSpecification(JobSpecification):
    """The Base polyaxonfile specification (parsing and validation of Polyaxonfiles/Configurations).

    SECTIONS:
        VERSION: defines the version of the file to be parsed and validated.
        LOGGING: defines the logging
        TAGS: defines the tags
        ENVIRONMENT: defines the run environment for experiment.
        DECLARATIONS: variables/modules that can be reused.
        BUILD: defines the build step where the user can set a docker image definition
        RUN: defines the run step where the user can run a command
    """
    _SPEC_KIND = BaseSpecification._EXPERIMENT  # pylint:disable=protected-access

    POSSIBLE_SECTIONS = JobSpecification.POSSIBLE_SECTIONS + (
        JobSpecification.DECLARATIONS,
        JobSpecification.MODEL,
        JobSpecification.TRAIN,
        JobSpecification.EVAL
    )
    REQUIRED_SECTIONS = BaseSpecification.REQUIRED_SECTIONS

    @cached_property
    def is_runnable(self):
        """Checks of the sections required to run experiment exist."""
        sections = set(self.validated_data.keys())
        condition = (self.RUN in sections or
                     {self.MODEL, self.TRAIN} <= sections or
                     {self.MODEL, self.EVAL} <= sections)
        if condition:
            return True
        return False

    @cached_property
    def model(self):
        return self.validated_data.get(self.MODEL, None)

    @cached_property
    def train(self):
        return self.validated_data.get(self.TRAIN, None)

    @cached_property
    def eval(self):
        return self.validated_data.get(self.EVAL, None)

    @cached_property
    def declarations(self):
        return self.parsed_data.get(self.DECLARATIONS, None)

    @cached_property
    def framework(self):
        if not self.environment:
            return None

        if self.environment.tensorflow:
            return Frameworks.TENSORFLOW

        if self.environment.horovod:
            return Frameworks.HOROVOD

        if self.environment.mxnet:
            return Frameworks.MXNET

        if self.environment.pytorch:
            return Frameworks.PYTORCH

    @cached_property
    def cluster_def(self):
        cluster = {
            TaskType.MASTER: 1,
        }
        is_distributed = False
        environment = self.environment

        if not environment:
            return cluster, is_distributed

        if environment.tensorflow:
            return TensorflowSpecification.get_cluster_def(
                cluster=cluster,
                framework_config=environment.tensorflow)
        if environment.horovod:
            return HorovodSpecification.get_cluster_def(
                cluster=cluster,
                framework_config=environment.horovod)
        if environment.mxnet:
            return MXNetSpecification.get_cluster_def(
                cluster=cluster,
                framework_config=environment.mxnet)
        if environment.pytorch:
            return PytorchSpecification.get_cluster_def(
                cluster=cluster,
                framework_config=environment.pytorch)

        # No specified framework, It should return default standalone mode cluster definition
        return cluster, is_distributed

    @cached_property
    def total_resources(self):
        environment = self.environment

        if not environment:
            return None

        cluster, is_distributed = self.cluster_def

        # Check if any framework is defined
        if environment.tensorflow:
            return TensorflowSpecification.get_total_resources(
                master_resources=self.master_resources,
                environment=environment,
                cluster=cluster,
                is_distributed=is_distributed
            )

        if environment.horovod:
            return HorovodSpecification.get_total_resources(
                master_resources=self.master_resources,
                environment=environment,
                cluster=cluster,
                is_distributed=is_distributed
            )

        if environment.mxnet:
            return MXNetSpecification.get_total_resources(
                master_resources=self.master_resources,
                environment=environment,
                cluster=cluster,
                is_distributed=is_distributed
            )

        if environment.pytorch:
            return PytorchSpecification.get_total_resources(
                master_resources=self.master_resources,
                environment=environment,
                cluster=cluster,
                is_distributed=is_distributed
            )

        # default value is the master resources
        return self.master_resources

    @cached_property
    def master_resources(self):
        return self.environment.resources if self.environment else None

    @cached_property
    def master_node_selectors(self):
        return self.environment.node_selectors if self.environment else None



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from tests.utils import assert_equal_dict

from polyaxon_schemas.rl.environments import GymEnvironmentConfig
from polyaxon_schemas.rl.explorations import (
    ConstantExplorationConfig,
    DecayExplorationConfig,
    GreedyExplorationConfig,
    OrnsteinUhlenbeckExplorationConfig,
    RandomDecayExplorationConfig,
    RandomExplorationConfig
)
from polyaxon_schemas.rl.memories import BatchMemoryConfig


class TestMemoryConfigs(TestCase):
    def test_latent_bridge_config(self):
        config_dict = {
            'env_id': 'CartPole-v0',
        }
        config = GymEnvironmentConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)


class TestEnvironmentConfigs(TestCase):
    def test_latent_bridge_config(self):
        config_dict = {
            'size': 500,
            'batch_size': 500,
        }
        config = BatchMemoryConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)


class TestExplorationConfigs(TestCase):
    def test_constant_exploration_config(self):
        config_dict = {
            'value': 0.8,
            'is_continuous': False
        }
        config = ConstantExplorationConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_greedy_exploration_config(self):
        config_dict = {
            'is_continuous': False
        }
        config = GreedyExplorationConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_random_exploration_config(self):
        config_dict = {
            'is_continuous': False
        }
        config = RandomExplorationConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_decay_exploration_config(self):
        config_dict = {
            'is_continuous': False,
            'exploration_rate': 0.15,
            'decay_type': 'polynomial_decay',
            'start_decay_at': 0,
            'stop_decay_at': 1e9,
            'decay_rate': 0.,
            'staircase': False,
            'decay_steps': 100000,
            'min_exploration_rate': 0
        }
        config = DecayExplorationConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_random_decay_exploration_config(self):
        config_dict = {
            'is_continuous': False,
            'num_actions': None,
            'decay_type': 'polynomial_decay',
            'start_decay_at': 0,
            'stop_decay_at': 1e9,
            'decay_rate': 0.,
            'staircase': False,
            'decay_steps': 10000,
            'min_exploration_rate': 0
        }
        config = RandomDecayExplorationConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

    def test_orsteinuhlenbeck_exploration_config(self):
        config_dict = {
            'is_continuous': True,
            'num_actions': 4,
            'sigma': 0.3,
            'mu': 0,
            'theta': 0.15
        }
        config = OrnsteinUhlenbeckExplorationConfig.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from tests.utils import assert_equal_feature_processors, assert_equal_layers

from polyaxon_schemas.processing.feature_processors import FeatureProcessorsConfig
from polyaxon_schemas.processing.image import (
    AdjustBrightnessConfig,
    AdjustContrastConfig,
    AdjustGammaConfig,
    AdjustHueConfig,
    AdjustSaturationConfig,
    CentralCropConfig,
    ConvertColorSpaceConfig,
    ConvertImagesDtypeConfig,
    DrawBoundingBoxesConfig,
    ExtractGlimpseConfig,
    FlipConfig,
    RandomCropConfig,
    ResizeConfig,
    Rotate90Config,
    StandardizationConfig,
    ToBoundingBoxConfig,
    TotalVariationConfig,
    TransposeConfig
)
from polyaxon_schemas.processing.pipelines import (
    BasePipelineConfig,
    ImageCaptioningPipelineConfig,
    ParallelTextPipelineConfig,
    TFRecordImagePipelineConfig,
    TFRecordSequencePipelineConfig,
    TFRecordSourceSequencePipelineConfig
)


class TestFeatureProcessorsConfigs(TestCase):
    def test_feature_processors(self):
        config_dict = {
            'image1': {
                'input_layers': ['image'],
                'output_layers': ['reshap_0'],
                'layers': [
                    {'Resize': {'height': 28, 'width': 28}},
                    {'Reshape': {'target_shape': [784]}}
                ]
            },
            'image2': {
                'input_layers': ['image'],
                'output_layers': ['reshap_0'],
                'layers': [
                    {'Standardization': {}},
                    {'Resize': {'height': 28, 'width': 28}},
                    {'Reshape': {'target_shape': [784]}}
                ]
            }
        }
        config = FeatureProcessorsConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert_equal_feature_processors(config_to_dict, config_dict)


class TestPipelinesConfigs(TestCase):
    def test_base_pipeline_config(self):
        config_dict = {
            'name': 'my_pipelne',
            'num_epochs': 10,
            'shuffle': True,
            'feature_processors': {
                'image': {
                    'input_layers': ['image'],
                    'output_layers': ['reshap_0'],
                    'layers': [
                        {'Resize': {'height': 28, 'width': 28}},
                        {'Reshape': {'target_shape': [784]}}
                    ]
                },
            }
        }
        config = BasePipelineConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert config_to_dict['name'] == config_dict['name']
        assert config_to_dict['num_epochs'] == config_dict['num_epochs']
        assert config_to_dict['shuffle'] == config_dict['shuffle']

        assert_equal_feature_processors(config_to_dict['feature_processors'],
                                        config_dict['feature_processors'])

    def test_tf_record_image_pipeline_config(self):
        config_dict = {
            'batch_size': 64,
            'num_epochs': 10,
            'shuffle': True,
            'data_files': ['train_data_file'],
            'meta_data_file': 'meta_data_file',
            'feature_processors': {
                'image': {
                    'input_layers': ['image'],
                    'output_layers': ['reshap_0'],
                    'layers': [
                        {'Resize': {'height': 28, 'width': 28}},
                        {'Reshape': {'target_shape': [784]}}
                    ]
                },
            }
        }

        config = TFRecordImagePipelineConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert config_to_dict['num_epochs'] == config_dict['num_epochs']
        assert config_to_dict['shuffle'] == config_dict['shuffle']
        assert config_to_dict['data_files'] == config_dict['data_files']
        assert config_to_dict['meta_data_file'] == config_dict['meta_data_file']

        assert_equal_feature_processors(config_to_dict['feature_processors'],
                                        config_dict['feature_processors'])

    def test_tf_record_sequence_pipeline_config(self):
        config_dict = {
            'batch_size': 64,
            'num_epochs': 10,
            'shuffle': True,
            'data_files': ['train_data_file'],
            'meta_data_file': 'meta_data_file',
        }

        config = TFRecordSequencePipelineConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert config_to_dict['num_epochs'] == config_dict['num_epochs']
        assert config_to_dict['shuffle'] == config_dict['shuffle']
        assert config_to_dict['data_files'] == config_dict['data_files']
        assert config_to_dict['meta_data_file'] == config_dict['meta_data_file']

    def test_parallel_text_pipeline_config(self):
        config_dict = {
            'batch_size': 64,
            'num_epochs': 10,
            'shuffle': True,
            'source_files': ['source_data_file'],
            'target_files': ['target_data_file'],
        }

        config = ParallelTextPipelineConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert config_to_dict['num_epochs'] == config_dict['num_epochs']
        assert config_to_dict['shuffle'] == config_dict['shuffle']
        assert config_to_dict['source_files'] == config_dict['source_files']
        assert config_to_dict['target_files'] == config_dict['target_files']
        assert config_to_dict['source_delimiter'] == ""
        assert config_to_dict['target_delimiter'] == ""

    def test_tf_record_source_sequence_pipeline_config(self):
        config_dict = {
            'batch_size': 64,
            'num_epochs': 10,
            'shuffle': True,
            'files': ['source_data_file'],
            'source_field': 'source',
            'target_field': 'target'
        }

        config = TFRecordSourceSequencePipelineConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert config_to_dict['num_epochs'] == config_dict['num_epochs']
        assert config_to_dict['shuffle'] == config_dict['shuffle']
        assert config_to_dict['files'] == config_dict['files']
        assert config_to_dict['source_field'] == config_dict['source_field']
        assert config_to_dict['target_field'] == config_dict['target_field']
        assert config_to_dict['source_delimiter'] == ""
        assert config_to_dict['target_delimiter'] == ""

    def test_image_captioning_pipeline_config(self):
        config_dict = {
            'batch_size': 64,
            'num_epochs': 10,
            'shuffle': True,
            'files': ['source_data_file'],
            'image_field': 'image/data',
            'image_format': 'jpg',
            'caption_ids_field': 'image/caption_ids',
            'caption_tokens_field': 'image/caption'
        }

        config = ImageCaptioningPipelineConfig.from_dict(config_dict)
        config_to_dict = config.to_dict()

        assert config_to_dict['num_epochs'] == config_dict['num_epochs']
        assert config_to_dict['shuffle'] == config_dict['shuffle']
        assert config_to_dict['files'] == config_dict['files']
        assert config_to_dict['image_field'] == config_dict['image_field']
        assert config_to_dict['image_format'] == config_dict['image_format']
        assert config_to_dict['caption_ids_field'] == config_dict['caption_ids_field']
        assert config_to_dict['caption_tokens_field'] == config_dict['caption_tokens_field']


class TestImageProcessingConfigs(TestCase):
    def test_resize_config(self):
        config_dict = {
            'height': 28,
            'width': 28,
            'method': 0,
            'align_corners': True,
            'name': 'Resize'
        }
        config = ResizeConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_central_crop_config(self):
        config_dict = {
            'central_fraction': 0.28,
            'name': 'CentralCrop'
        }

        config = CentralCropConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_random_crop_config(self):
        config_dict = {
            'height': 28,
            'width': 28,
            'name': 'RandomCrop'
        }

        config = RandomCropConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_extract_glimpse_config(self):
        config_dict = {
            'size': [1, 1],
            'offsets': [1, 1],
            'centered': True,
            'normalized': True,
            'uniform_noise': True,
            'name': 'ExtractGlimpse'
        }

        config = ExtractGlimpseConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_to_bounding_box_config(self):
        config_dict = {
            'offset_height': 1,
            'offset_width': 1,
            'target_height': 10,
            'target_width': 10,
            'method': 'crop',
            'name': 'ToBoundingBox'
        }

        config = ToBoundingBoxConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_flip_config(self):
        config_dict = {
            'axis': 0,
            'is_random': False,
            'seed': None,
            'name': 'Flip'
        }

        config = FlipConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_transpose_config(self):
        config_dict = {
            'name': 'Transpose'
        }

        config = TransposeConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_rotate_config(self):
        config_dict = {
            'k': 0,
            'is_random': False,
            'seed': None,
            'name': 'Rotate90'
        }

        config = Rotate90Config.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_convert_color_space_config(self):
        config_dict = {
            'from_space': 'rgb',
            'to_space': 'grayscale',
            'name': 'ConvertColorSpace'
        }

        config = ConvertColorSpaceConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_convert_image_dtype_config(self):
        config_dict = {
            'dtype': 'float32',
            'saturate': True,
            'name': 'ConvertImagesDtype'
        }

        config = ConvertImagesDtypeConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_adjust_brightness_config(self):
        config_dict = {
            'delta': 1.3,
            'is_random': True,
            'seed': 1000,
            'name': 'AdjustBrightness'
        }

        config = AdjustBrightnessConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_adjust_contrast_config(self):
        config_dict = {
            'contrast_factor': 1.3,
            'contrast_factor_max': None,
            'is_random': False,
            'seed': 1000,
            'name': 'AdjustContrast'
        }

        config = AdjustContrastConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_adjust_hue_config(self):
        config_dict = {
            'delta': 0.3,
            'is_random': True,
            'seed': 1000,
            'name': 'AdjustHue'
        }

        config = AdjustHueConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_adjust_saturation_config(self):
        config_dict = {
            'saturation_factor': 0.3,
            'saturation_factor_max': None,
            'is_random': True,
            'seed': 1000,
            'name': 'AdjustSaturation'
        }

        config = AdjustSaturationConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_adjust_gamma_config(self):
        config_dict = {
            'gamma': 0.3,
            'gain': 1,
            'name': 'AdjustGamma'
        }

        config = AdjustGammaConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_standardization_config(self):
        config_dict = {
            'name': 'Standardization'
        }

        config = StandardizationConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_draw_bounding_boxes_config(self):
        config_dict = {
            'boxes': [0, 3, 3],
            'name': 'DrawBoundingBoxes'
        }

        config = DrawBoundingBoxesConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)

    def test_total_variation_config(self):
        config_dict = {
            'name': 'TotalVariation'
        }

        config = TotalVariationConfig.from_dict(config_dict)
        assert_equal_layers(config, config_dict)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from polyaxon_schemas.log_handler import LogHandlerConfig


class TestLogHandlerConfig(TestCase):
    def test_log_handler_config(self):
        config_dict = {'dns': 'https//foo:bar',
                       'environment': 'staging',
                       'tags': {}}
        config = LogHandlerConfig.from_dict(config_dict)
        assert config.to_dict() == config_dict



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import datetime

from unittest import TestCase

from polyaxon_schemas.utils import humanize_timedelta, humanize_timesince, local_now, to_percentage


class HumanizeTimesinceTest(TestCase):
    """A test case for humanize timesince"""

    def test_humanize_timesince(self):
        self.assertEqual(humanize_timesince(local_now()), 'a few seconds ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(minutes=1)),
                         '1 minute ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(minutes=10)),
                         '10 minutes ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=1)),
                         '1 hour ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=10)),
                         '10 hours ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=24)),
                         '1 day ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=72)),
                         '3 days ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=168)),
                         '1 week ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(weeks=1)),
                         '1 week ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(weeks=3)),
                         '3 weeks ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(weeks=53)),
                         '1 year ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(days=365)),
                         '1 year ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(days=800)),
                         '2 years ago')

    def test_humanize_times_in_the_future(self):
        self.assertEqual(humanize_timesince(local_now() + datetime.timedelta(minutes=1)),
                         'a few seconds ago')

    def test_humanize_timesince_few_seconds(self):
        self.assertEqual(u'Last update: ' + humanize_timesince(local_now()),
                         u'Last update: a few seconds ago')


class HumanizeTimeDeltaTest(TestCase):
    """A test case for the `humanize_timedelta`."""
    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (7200, '2h'),
            (36, '36s'),
            (3600, '1h'),
            (3800, '1h 3m'),
            (33000, '9h 10m'),
            (720000, '8d 8h'),
            (1000000, '11d 13h 46m'),
        ]
        for value, expected in test_data:
            result = humanize_timedelta(value)
            self.assertEqual(result, expected)


class ToPercentageTest(TestCase):
    """A test case for the `to_percentage`."""
    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (0, '0%'),
            (0.25, '25%'),
            (-0.25, '-25%'),
            (12, '1200%'),
            (0.123, '12.3%'),
            (0.12345, '12.35%'),
            (0.12001, '12%'),
            (0.12101, '12.1%'),
            ('0', '0%'),
            ('0.25', '25%'),
        ]
        for value, expected in test_data:
            result = to_percentage(value)
            self.assertEqual(result, expected)

    def test_raises_value_error_for_invalid_types(self):
        with self.assertRaises(ValueError):
            to_percentage('foo')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import base64

from marshmallow import Schema, fields, post_dump, post_load

from polyaxon_schemas.base import BaseConfig


class LogHandlerSchema(Schema):
    dns = fields.Str(allow_none=True)
    environment = fields.Str()
    tags = fields.Dict()

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return LogHandlerConfig(**data)

    @post_dump
    def unmake(self, data):
        return LogHandlerConfig.remove_reduced_attrs(data)


class LogHandlerConfig(BaseConfig):
    SCHEMA = LogHandlerSchema
    IDENTIFIER = 'log_handler'

    def __init__(self, environment, tags, dns=None):
        self.dns = dns
        self.environment = environment
        self.tags = tags

    @property
    def decoded_dns(self):
        if self.dns:
            return base64.b64decode(self.dns.encode('utf-8')).decode('utf-8')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import Schema, fields, post_dump, post_load, validate

from polyaxon_schemas.base import BaseConfig


class LoggingSchema(Schema):
    level = fields.Str(allow_none=True, validate=validate.OneOf(
        ['INFO', 'DEBUG', 'WARN', 'ERROR', 'FATAL']))
    formatter = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return LoggingConfig(**data)

    @post_dump
    def unmake(self, data):
        return LoggingConfig.remove_reduced_attrs(data)


class LoggingConfig(BaseConfig):
    SCHEMA = LoggingSchema
    IDENTIFIER = 'logging'

    def __init__(self, level='INFO', formatter=None):
        self.level = level
        self.formatter = formatter



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import fields, post_dump, post_load

from polyaxon_schemas.constraints import ConstraintSchema
from polyaxon_schemas.initializations import InitializerSchema, ZerosInitializerConfig
from polyaxon_schemas.layers.base import BaseLayerConfig, BaseLayerSchema
from polyaxon_schemas.regularizations import RegularizerSchema
from polyaxon_schemas.utils import ObjectOrListObject


class LeakyReLUSchema(BaseLayerSchema):
    alpha = fields.Float(default=0.3, missing=0.3)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return LeakyReLUConfig(**data)

    @post_dump
    def unmake(self, data):
        return LeakyReLUConfig.remove_reduced_attrs(data)


class LeakyReLUConfig(BaseLayerConfig):
    """Leaky version of a Rectified Linear Unit.

    It allows a small gradient when the unit is not active:
    `f(x) = alpha * x for x < 0`,
    `f(x) = x for x >= 0`.

    Args:
        alpha: float >= 0. Negative slope coefficient.

    Input shape:
        Arbitrary. Use the keyword argument `input_shape`
        (tuple of integers, does not include the samples axis)
        when using this layer as the first layer in a model.

    Output shape:
        Same shape as the input.

    Polyaxonfile usage:

    ```yaml
    LeakyReLU:
        alpha: 0.2
    ```
    """
    IDENTIFIER = 'LeakyReLU'
    SCHEMA = LeakyReLUSchema

    def __init__(self, alpha=0.3, **kwargs):
        super(LeakyReLUConfig, self).__init__(**kwargs)
        self.alpha = alpha


class PReLUSchema(BaseLayerSchema):
    alpha_initializer = fields.Nested(InitializerSchema, default=None, missing=None)
    alpha_regularizer = fields.Nested(RegularizerSchema, default=None, missing=None)
    alpha_constraint = fields.Nested(ConstraintSchema, default=None, missing=None)
    shared_axes = ObjectOrListObject(fields.Int, default=None, missing=None)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return PReLUConfig(**data)

    @post_dump
    def unmake(self, data):
        return PReLUConfig.remove_reduced_attrs(data)


class PReLUConfig(BaseLayerConfig):
    """Parametric Rectified Linear Unit.

    It follows:
    `f(x) = alpha * x for x < 0`,
    `f(x) = x for x >= 0`,
    where `alpha` is a learned array with the same shape as x.

    Args:
        alpha_initializer: initializer function for the weights.
        alpha_regularizer: regularizer for the weights.
        alpha_constraint: constraint for the weights.
        shared_axes: the axes along which to share learnable
            parameters for the activation function.
            For example, if the incoming feature maps
            are from a 2D convolution
            with output shape `(batch, height, width, channels)`,
            and you wish to share parameters across space
            so that each filter only has one set of parameters,
            set `shared_axes=[1, 2]`.

    Input shape:
        Arbitrary. Use the keyword argument `input_shape`
        (tuple of integers, does not include the samples axis)
        when using this layer as the first layer in a model.

    Output shape:
        Same shape as the input.

    Polyaxonfile usage:

    ```yaml
    PReLU:
      alpha_initializer:
        ZerosInitializer:
      alpha_regularizer:
        L2:
         l: 0.01
      shared_axes: [1, 2]
    ```
    """
    IDENTIFIER = 'PReLU'
    SCHEMA = PReLUSchema

    def __init__(self,
                 alpha_initializer=ZerosInitializerConfig(),
                 alpha_regularizer=None,
                 alpha_constraint=None,
                 shared_axes=None,
                 **kwargs):
        super(PReLUConfig, self).__init__(**kwargs)
        self.alpha_initializer = alpha_initializer
        self.alpha_regularizer = alpha_regularizer
        self.alpha_constraint = alpha_constraint
        self.shared_axes = shared_axes


class ELUSchema(BaseLayerSchema):
    alpha = fields.Float(default=1.0, missing=1.0)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ELUConfig(**data)

    @post_dump
    def unmake(self, data):
        return ELUConfig.remove_reduced_attrs(data)


class ELUConfig(BaseLayerConfig):
    """Exponential Linear Unit.

    It follows:
    `f(x) =  alpha * (exp(x) - 1.) for x < 0`,
    `f(x) = x for x >= 0`.

    Args:
        alpha: scale for the negative factor.

    Input shape:
        Arbitrary. Use the keyword argument `input_shape`
        (tuple of integers, does not include the samples axis)
        when using this layer as the first layer in a model.

    Output shape:
        Same shape as the input.

    Polyaxonfile usage:

    ```yaml
    ELU:
      alpha:0.1
    ```
    """
    IDENTIFIER = 'ELU'
    SCHEMA = ELUSchema

    def __init__(self, alpha=0.1, **kwargs):
        super(ELUConfig, self).__init__(**kwargs)
        self.alpha = alpha


class ThresholdedReLUSchema(BaseLayerSchema):
    theta = fields.Float(default=1.0, missing=1.0)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ThresholdedReLUConfig(**data)

    @post_dump
    def unmake(self, data):
        return ThresholdedReLUConfig.remove_reduced_attrs(data)


class ThresholdedReLUConfig(BaseLayerConfig):
    """Thresholded Rectified Linear Unit.

    It follows:
    `f(x) = x for x > theta`,
    `f(x) = 0 otherwise`.

    Args:
        theta: float >= 0. Threshold location of activation.

    Input shape:
        Arbitrary. Use the keyword argument `input_shape`
        (tuple of integers, does not include the samples axis)
        when using this layer as the first layer in a model.

    Output shape:
        Same shape as the input.

    Polyaxonfile usage:

    ```yaml
    ThresholdedReLU:
      theta:0.1
    ```
    """
    IDENTIFIER = 'ThresholdedReLU'
    SCHEMA = ThresholdedReLUSchema

    def __init__(self, theta=1.0, **kwargs):
        super(ThresholdedReLUConfig, self).__init__(**kwargs)
        self.theta = theta



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import (
    Schema,
    ValidationError,
    fields,
    post_dump,
    post_load,
    validate,
    validates_schema
)

from polyaxon_schemas.base import BaseConfig


def validate_image(image):
    if not image:
        raise ValidationError('Invalid docker image `{}`'.format(image))
    tagged_image = image.split(':')
    if len(tagged_image) > 2:
        raise ValidationError('Invalid docker image `{}`'.format(image))


class BuildSchema(Schema):
    image = fields.Str()
    build_steps = fields.List(fields.Str(), allow_none=True)
    env_vars = fields.List(fields.List(fields.Raw(), validate=validate.Length(equal=2)),
                           allow_none=True)
    git = fields.Str(allow_none=True)
    nocache = fields.Boolean(allow_none=True)
    commit = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return BuildConfig(**data)

    @post_dump
    def unmake(self, data):
        return BuildConfig.remove_reduced_attrs(data)

    @validates_schema
    def validate_image(self, data):
        """Validates docker image structure"""
        validate_image(data.get('image'))


class BuildConfig(BaseConfig):
    SCHEMA = BuildSchema
    IDENTIFIER = 'build'
    REDUCED_ATTRIBUTES = ['build_steps', 'env_vars', 'git', 'nocache', 'commit']

    def __init__(self, image, build_steps=None, env_vars=None, git=None, nocache=None, commit=None):
        validate_image(image)
        self.image = image
        self.build_steps = build_steps
        self.env_vars = env_vars
        self.git = git
        self.nocache = nocache
        self.commit = commit

    @property
    def image_tag(self):
        tagged_image = self.image.split(':')
        if len(tagged_image) == 1:
            return 'latest'

        return tagged_image[1]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import Schema, fields, post_dump, post_load

from polyaxon_schemas.base import BaseConfig, BaseMultiSchema


class MaxNormSchema(Schema):
    max_value = fields.Int(default=2, missing=2)
    axis = fields.Int(default=0, missing=0)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return MaxNormConfig(**data)

    @post_dump
    def unmake(self, data):
        return MaxNormConfig.remove_reduced_attrs(data)


class MaxNormConfig(BaseConfig):
    """MaxNorm weight constraint.

    Constrains the weights incident to each hidden unit
    to have a norm less than or equal to a desired value.

    Args:
        m: the maximum norm for the incoming weights.
        axis: integer, axis along which to calculate weight norms.
            For instance, in a `Dense` layer the weight matrix
            has shape `(input_dim, output_dim)`,
            set `axis` to `0` to constrain each weight vector
            of length `(input_dim,)`.
            In a `Conv2D` layer with `data_format="channels_last"`,
            the weight tensor has shape
            `(rows, cols, input_depth, output_depth)`,
            set `axis` to `[0, 1, 2]`
            to constrain the weights of each filter tensor of size
            `(rows, cols, input_depth)`.

    References:
        - [Dropout: A Simple Way to Prevent Neural Networks from Overfitting
          Srivastava, Hinton, et al.
          2014](http://www.cs.toronto.edu/~rsalakhu/papers/srivastava14a.pdf)

    Polyaxonfile usage:

    Using the default values

    ```yaml
    MaxNorm:
    ```

    Using custom values

    ```yaml
    MaxNorm:
      max_value: 3
      axis: 0
    ```

    Example with layer

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint: MaxNorm
    ```

    or

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint:
        MaxNorm:
          max_value: 3
    ```
    """
    IDENTIFIER = 'MaxNorm'
    SCHEMA = MaxNormSchema

    def __init__(self, max_value=2, axis=0):
        self.max_value = max_value
        self.axis = axis


class NonNegSchema(Schema):
    w = fields.Float()

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return NonNegConfig(**data)

    @post_dump
    def unmake(self, data):
        return NonNegConfig.remove_reduced_attrs(data)


class NonNegConfig(BaseConfig):
    """Constrains the weights to be non-negative.

    Polyaxonfile usage:

    ```yaml
    NonNeg:
      w: 0.2
    ```

    Example with layer:

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint:
        NonNeg:
          w: 0.2
    ```
    """
    IDENTIFIER = 'NonNeg'
    SCHEMA = NonNegSchema

    def __init__(self, w):
        self.w = w


class UnitNormSchema(Schema):
    axis = fields.Int(default=0, missing=0)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return UnitNormConfig(**data)

    @post_dump
    def unmake(self, data):
        return UnitNormConfig.remove_reduced_attrs(data)


class UnitNormConfig(BaseConfig):
    """Constrains the weights incident to each hidden unit to have unit norm.

    Args:
        axis: integer, axis along which to calculate weight norms.
            For instance, in a `Dense` layer the weight matrix
            has shape `(input_dim, output_dim)`,
            set `axis` to `0` to constrain each weight vector
            of length `(input_dim,)`.
            In a `Conv2D` layer with `data_format="channels_last"`,
            the weight tensor has shape
            `(rows, cols, input_depth, output_depth)`,
            set `axis` to `[0, 1, 2]`
            to constrain the weights of each filter tensor of size
            `(rows, cols, input_depth)`.

    Polyaxonfile usage:

    Using the default values

    ```yaml
    UnitNorm:
    ```

    Using custom values

    ```yaml
    UnitNorm:
      axis: 1
    ```

    Example with layer

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint: UnitNorm
    ```

    or

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint:
        UnitNorm:
          axis: 1
    ```
    """
    IDENTIFIER = 'UnitNorm'
    SCHEMA = UnitNormSchema

    def __init__(self, axis=0):
        self.axis = axis


class MinMaxNormSchema(Schema):
    min_value = fields.Float(default=0., missing=0.)
    max_value = fields.Float(default=1., missing=1.)
    rate = fields.Float(default=1., missing=1.)
    axis = fields.Int(default=0, missing=0)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return MinMaxNormConfig(**data)

    @post_dump
    def unmake(self, data):
        return MinMaxNormConfig.remove_reduced_attrs(data)


class MinMaxNormConfig(BaseConfig):
    """MinMaxNorm weight constraint.

    Constrains the weights incident to each hidden unit
    to have the norm between a lower bound and an upper bound.

    Args:
        min_value: the minimum norm for the incoming weights.
        max_value: the maximum norm for the incoming weights.
        rate: rate for enforcing the constraint: weights will be
            rescaled to yield
            `(1 - rate) * norm + rate * norm.clip(min_value, max_value)`.
            Effectively, this means that rate=1.0 stands for strict
            enforcement of the constraint, while rate<1.0 means that
            weights will be rescaled at each step to slowly move
            towards a value inside the desired interval.
        axis: integer, axis along which to calculate weight norms.
            For instance, in a `Dense` layer the weight matrix
            has shape `(input_dim, output_dim)`,
            set `axis` to `0` to constrain each weight vector
            of length `(input_dim,)`.
            In a `Conv2D` layer with `dim_ordering="channels_last"`,
            the weight tensor has shape
            `(rows, cols, input_depth, output_depth)`,
            set `axis` to `[0, 1, 2]`
            to constrain the weights of each filter tensor of size
            `(rows, cols, input_depth)`.

    Polyaxonfile usage:

    Using the default values

    ```yaml
    MinMaxNorm:
    ```

    Using custom values

    ```yaml
    MinMaxNorm:
      min_value: 0.1
      max_value: 0.8
      rate: 0.9
      axis: 0
    ```

    Example with layer

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint: MinMaxNorm
    ```

    or

    ```yaml
    Conv2D:
      filters: 10
      kernel_size: 8
      kernel_constraint:
        MinMaxNorm:
          min_value: 0.1
          max_value: 0.8
          rate: 0.9
          axis: 0
    ```
    """
    IDENTIFIER = 'MinMaxNorm'
    SCHEMA = MinMaxNormSchema

    def __init__(self, min_value=0.0, max_value=1.0, rate=1.0, axis=0):
        self.min_value = min_value
        self.max_value = max_value
        self.rate = rate
        self.axis = axis


class ConstraintSchema(BaseMultiSchema):
    __multi_schema_name__ = 'constraint'
    __configs__ = {
        MaxNormConfig.IDENTIFIER: MaxNormConfig,
        NonNegConfig.IDENTIFIER: NonNegConfig,
        UnitNormConfig.IDENTIFIER: UnitNormConfig,
        MinMaxNormConfig.IDENTIFIER: MinMaxNormConfig,
    }



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import six

from polyaxon_schemas.environments import PodResourcesConfig
from polyaxon_schemas.polyaxonfile.specification.utils import (
    get_task_configs,
    get_task_job_node_selectors,
    get_task_job_resources
)
from polyaxon_schemas.utils import TaskType


class DistributedSpecificationInterface(object):
    TASK_WORKER = TaskType.WORKER
    TASK_PS = TaskType.PS

    @staticmethod
    def has_framework_environment(environment):
        return False

    @staticmethod
    def get_framework_environment(environment):
        return None

    @staticmethod
    def get_cluster_def(cluster, framework_config):
        pass

    @classmethod
    def get_worker_configs(cls, environment, cluster, is_distributed):
        framework_environment = cls.get_framework_environment(environment=environment)
        if not framework_environment:
            return {}

        return get_task_configs(cluster=cluster,
                                is_distributed=is_distributed,
                                configs=framework_environment.worker_configs,
                                default_config=framework_environment.default_worker_config,
                                task_type=cls.TASK_WORKER)

    @classmethod
    def get_ps_configs(cls, environment, cluster, is_distributed):
        framework_environment = cls.get_framework_environment(environment=environment)
        if not framework_environment:
            return {}

        return get_task_configs(cluster=cluster,
                                is_distributed=is_distributed,
                                configs=framework_environment.ps_configs,
                                default_config=framework_environment.default_ps_config,
                                task_type=cls.TASK_PS)

    @classmethod
    def get_worker_resources(cls, environment, cluster, is_distributed):
        framework_environment = cls.get_framework_environment(environment=environment)
        if not framework_environment:
            return None

        return get_task_job_resources(
            cluster=cluster,
            is_distributed=is_distributed,
            resources=framework_environment.worker_resources,
            default_resources=framework_environment.default_worker_resources,
            task_type=cls.TASK_WORKER)

    @classmethod
    def get_ps_resources(cls, environment, cluster, is_distributed):
        framework_environment = cls.get_framework_environment(environment=environment)
        if not framework_environment:
            return None
        return get_task_job_resources(
            cluster=cluster,
            is_distributed=is_distributed,
            resources=framework_environment.ps_resources,
            default_resources=framework_environment.default_ps_resources,
            task_type=cls.TASK_PS)

    @classmethod
    def get_total_resources(cls, master_resources, environment, cluster, is_distributed):
        pass

    @classmethod
    def get_worker_node_selectors(cls, environment, cluster, is_distributed):
        framework_environment = cls.get_framework_environment(environment=environment)
        if not framework_environment:
            return {}

        return get_task_job_node_selectors(
            cluster=cluster,
            is_distributed=is_distributed,
            node_selectors=framework_environment.worker_node_selectors,
            default_node_selectors=framework_environment.default_worker_node_selectors,
            task_type=cls.TASK_WORKER)

    @classmethod
    def get_ps_node_selectors(cls, environment, cluster, is_distributed):
        framework_environment = cls.get_framework_environment(environment=environment)
        if not framework_environment:
            return {}

        return get_task_job_node_selectors(
            cluster=cluster,
            is_distributed=is_distributed,
            node_selectors=framework_environment.ps_node_selectors,
            default_node_selectors=framework_environment.default_ps_node_selectors,
            task_type=cls.TASK_PS)


class TensorflowSpecification(DistributedSpecificationInterface):
    @staticmethod
    def has_framework_environment(environment):
        return environment is not None and environment.tensorflow is not None

    @classmethod
    def get_framework_environment(cls, environment):
        if not cls.has_framework_environment(environment=environment):
            return None
        return environment.tensorflow

    @staticmethod
    def get_cluster_def(cluster, framework_config):
        is_distributed = False
        if not framework_config:
            return cluster, is_distributed

        cluster[TaskType.WORKER] = framework_config.n_workers
        cluster[TaskType.PS] = framework_config.n_ps
        if framework_config.n_workers != 0 or framework_config.n_ps != 0:
            is_distributed = True

        return cluster, is_distributed

    @classmethod
    def get_total_resources(cls, master_resources, environment, cluster, is_distributed):
        worker_resources = cls.get_worker_resources(
            environment=environment,
            cluster=cluster,
            is_distributed=is_distributed,
        )
        ps_resources = cls.get_ps_resources(
            environment=environment,
            cluster=cluster,
            is_distributed=is_distributed,
        )
        if not any([master_resources, worker_resources, ps_resources]):
            return None

        total_resources = PodResourcesConfig()

        if master_resources:
            total_resources += master_resources

        for w_resources in six.itervalues(worker_resources or {}):
            total_resources += w_resources

        for p_resources in six.itervalues(ps_resources or {}):
            total_resources += p_resources

        return total_resources.to_dict()


class HorovodSpecification(DistributedSpecificationInterface):
    @staticmethod
    def has_framework_environment(environment):
        return environment is not None and environment.horovod is not None

    @classmethod
    def get_framework_environment(cls, environment):
        if not cls.has_framework_environment(environment=environment):
            return None
        return environment.horovod

    @staticmethod
    def get_cluster_def(cluster, framework_config):
        is_distributed = False
        if not framework_config:
            return cluster, is_distributed

        cluster[TaskType.WORKER] = framework_config.n_workers
        if framework_config.n_workers != 0:
            is_distributed = True

        return cluster, is_distributed

    @classmethod
    def get_total_resources(cls, master_resources, environment, cluster, is_distributed):
        worker_resources = cls.get_worker_resources(
            environment=environment,
            cluster=cluster,
            is_distributed=is_distributed,
        )

        if not any([master_resources, worker_resources]):
            return None

        total_resources = PodResourcesConfig()

        if master_resources:
            total_resources += master_resources

        for w_resources in six.itervalues(worker_resources or {}):
            total_resources += w_resources

        return total_resources.to_dict()


class PytorchSpecification(DistributedSpecificationInterface):
    @staticmethod
    def has_framework_environment(environment):
        return environment is not None and environment.pytorch is not None

    @classmethod
    def get_framework_environment(cls, environment):
        if not cls.has_framework_environment(environment=environment):
            return None
        return environment.pytorch

    @staticmethod
    def get_cluster_def(cluster, framework_config):
        is_distributed = False
        if not framework_config:
            return cluster, is_distributed

        cluster[TaskType.WORKER] = framework_config.n_workers
        if framework_config.n_workers != 0:
            is_distributed = True

        return cluster, is_distributed

    @classmethod
    def get_total_resources(cls, master_resources, environment, cluster, is_distributed):
        worker_resources = cls.get_worker_resources(
            environment=environment,
            cluster=cluster,
            is_distributed=is_distributed,
        )

        if not any([master_resources, worker_resources]):
            return None

        total_resources = PodResourcesConfig()

        if master_resources:
            total_resources += master_resources

        for w_resources in six.itervalues(worker_resources or {}):
            total_resources += w_resources

        return total_resources.to_dict()


class MXNetSpecification(DistributedSpecificationInterface):
    TASK_PS = TaskType.SERVER

    @staticmethod
    def has_framework_environment(environment):
        return environment is not None and environment.mxnet is not None

    @classmethod
    def get_framework_environment(cls, environment):
        if not cls.has_framework_environment(environment=environment):
            return None
        return environment.mxnet

    @staticmethod
    def get_cluster_def(cluster, framework_config):
        is_distributed = False
        if not framework_config:
            return cluster, is_distributed

        cluster[TaskType.WORKER] = framework_config.n_workers
        cluster[TaskType.SERVER] = framework_config.n_ps
        if framework_config.n_workers != 0 or framework_config.n_ps != 0:
            is_distributed = True

        return cluster, is_distributed

    @classmethod
    def get_total_resources(cls, master_resources, environment, cluster, is_distributed):
        worker_resources = cls.get_worker_resources(
            environment=environment,
            cluster=cluster,
            is_distributed=is_distributed,
        )
        ps_resources = cls.get_ps_resources(
            environment=environment,
            cluster=cluster,
            is_distributed=is_distributed,
        )
        if not any([master_resources, worker_resources, ps_resources]):
            return None

        total_resources = PodResourcesConfig()

        if master_resources:
            total_resources += master_resources

        for w_resources in six.itervalues(worker_resources or {}):
            total_resources += w_resources

        for p_resources in six.itervalues(ps_resources or {}):
            total_resources += p_resources

        return total_resources.to_dict()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import copy

from polyaxon_schemas.build import BuildConfig
from polyaxon_schemas.environments import EnvironmentConfig
from polyaxon_schemas.eval import EvalConfig
from polyaxon_schemas.hptuning import HPTuningConfig
from polyaxon_schemas.logging import LoggingConfig
from polyaxon_schemas.models import ModelConfig
from polyaxon_schemas.run_exec import RunConfig
from polyaxon_schemas.train import TrainConfig


def validate_headers(spec, data):
    """Validates headers data and creates the config objects"""
    validated_data = {
        spec.VERSION: data[spec.VERSION],
        spec.KIND: data[spec.KIND],
    }

    if data.get(spec.LOGGING):
        validated_data[spec.LOGGING] = LoggingConfig.from_dict(
            data[spec.LOGGING])

    if data.get(spec.TAGS):
        validated_data[spec.TAGS] = data[spec.TAGS]

    if data.get(spec.HP_TUNING):
        validated_data[spec.HP_TUNING] = HPTuningConfig.from_dict(
            data[spec.HP_TUNING])

    return validated_data


def validate(spec, data):
    """Validates the data and creates the config objects"""
    data = copy.deepcopy(data)
    validated_data = {}

    def add_validated_section(section, config):
        if data.get(section):
            validated_data[section] = config.from_dict(data[section])

    add_validated_section(spec.ENVIRONMENT, EnvironmentConfig)
    add_validated_section(spec.BUILD, BuildConfig)
    add_validated_section(spec.RUN, RunConfig)
    add_validated_section(spec.MODEL, ModelConfig)
    add_validated_section(spec.TRAIN, TrainConfig)
    add_validated_section(spec.EVAL, EvalConfig)

    return validated_data



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from marshmallow import fields, post_dump, post_load, validate

from polyaxon_schemas.layers.base import BaseLayerConfig, BaseLayerSchema
from polyaxon_schemas.utils import DType

# pylint:disable=too-many-lines


class ResizeSchema(BaseLayerSchema):
    height = fields.Int()
    width = fields.Int()
    method = fields.Int(allow_none=True, validate=validate.Range(min=0, max=4))
    align_corners = fields.Bool(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ResizeConfig(**data)

    @post_dump
    def unmake(self, data):
        return ResizeConfig.remove_reduced_attrs(data)


class ResizeConfig(BaseLayerConfig):
    """Resize `images` to `size` using the specified `method`.
    (A mirror to tf.image resize_images and resize_image_with_crop_or_pad)

    - If method is None: Resizes an image to a target width and height by either centrally
    cropping the image or padding it evenly with zeros.
    If `width` or `height` is greater than the specified `target_width` or
    `target_height` respectively, this op centrally crops along that dimension.
    If `width` or `height` is smaller than the specified `target_width` or
    `target_height` respectively, this op centrally pads with 0 along that dimension.

    - If method is not None: the resized images will be distorted if their original aspect
    ratio is not the same as `size`.

    `method` can be one of:

    *   `None` no distortion.
    *   `ResizeMethod.BILINEAR`: [Bilinear interpolation.](https://en.wikipedia.org/wiki/Bilinear_interpolation)  # noqa
    *   `ResizeMethod.NEAREST_NEIGHBOR`: [Nearest neighbor interpolation.](https://en.wikipedia.org/wiki/Nearest-neighbor_interpolation)  # noqa
    *   `ResizeMethod.BICUBIC`: [Bicubic interpolation.](https://en.wikipedia.org/wiki/Bicubic_interpolation)  # noqa
    *   `ResizeMethod.AREA`: Area interpolation.

    Args:
        height: int32 Target height.
        width: int32 Target width.
        method: ResizeMethod.  Defaults to `ResizeMethod.BILINEAR`.
            Possible values: BILINEAR, NEAREST_NEIGHBOR, BICUBIC, AREA
        align_corners: bool. If true, exactly align all 4 corners of the input and output.
            Only used if method is not None. Defaults to `false`.

    Raises:
        ValueError: if the shape of `images` is incompatible with the
            shape arguments to this function.
        ValueError: if `size` has invalid shape or type.
        ValueError: if an unsupported resize method is specified.

    Returns:
        If `images` was 4-D, a 4-D float Tensor of shape
        `[batch, new_height, new_width, channels]`.
        If `images` was 3-D, a 3-D float Tensor of shape
        `[new_height, new_width, channels]`.

    Polyaxonfile usage:

    ```yaml
    - Resize:
        height: 10
        width: 10
        method: 0
    ```
    """
    SCHEMA = ResizeSchema
    IDENTIFIER = 'Resize'

    def __init__(self, height, width, method=None, align_corners=False, **kwargs):
        super(ResizeConfig, self).__init__(**kwargs)
        self.height = height
        self.width = width
        self.method = method
        self.align_corners = align_corners


class CentralCropSchema(BaseLayerSchema):
    central_fraction = fields.Float(validate=validate.Range(0., 1.))
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return CentralCropConfig(**data)

    @post_dump
    def unmake(self, data):
        return CentralCropConfig.remove_reduced_attrs(data)


class CentralCropConfig(BaseLayerConfig):
    """Crop the central region of the image.
    (A mirror to tf.image central_crop)

    Remove the outer parts of an image but retain the central region of the image
    along each dimension. If we specify central_fraction = 0.5, this function
    returns the region marked with "X" in the below diagram.

    ```
     --------
    |........|
    |..XXXX..|
    |..XXXX..|
    |........|   where "X" is the central 50% of the image.
     --------
    ```

    Args:
        central_fraction: float (0, 1], fraction of size to crop

    Raises:
        ValueError: if central_crop_fraction is not within (0, 1].

    Returns:
        If `images` was 4-D, a 4-D float Tensor of shape
        `[batch, new_height, new_width, channels]`.
        If `images` was 3-D, a 3-D float Tensor of shape
        `[new_height, new_width, channels]`.

    Polyaxonfile usage:

    ```yaml
    - CentralCrop:
        central_fraction: 0.5
    ```
    """
    SCHEMA = CentralCropSchema
    IDENTIFIER = 'CentralCrop'

    def __init__(self, central_fraction, **kwargs):
        super(CentralCropConfig, self).__init__(**kwargs)
        self.central_fraction = central_fraction


class RandomCropSchema(BaseLayerSchema):
    height = fields.Int()
    width = fields.Int()
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return RandomCropConfig(**data)

    @post_dump
    def unmake(self, data):
        return RandomCropConfig.remove_reduced_attrs(data)


class RandomCropConfig(BaseLayerConfig):
    """Randomly crops an image/images to a given size.

    Args:
        height: `float`. The height to crop to.
        width: `float`. The width to crop to.

    Returns:
        If `images` was 4-D, a 4-D float Tensor of shape
        `[batch, new_height, new_width, channels]`.
        If `images` was 3-D, a 3-D float Tensor of shape
        `[new_height, new_width, channels]`.

    Polyaxonfile usage:

    ```yaml
    - RandomCrop:
        height: 10
        width: 10
    ```
    """
    SCHEMA = RandomCropSchema
    IDENTIFIER = 'RandomCrop'

    def __init__(self, height, width, **kwargs):
        super(RandomCropConfig, self).__init__(**kwargs)
        self.height = height
        self.width = width


class ExtractGlimpseSchema(BaseLayerSchema):
    size = fields.List(fields.Int(), validate=validate.Length(equal=2))
    offsets = fields.List(fields.Int(), validate=validate.Length(equal=2))
    centered = fields.Bool(allow_none=True)
    normalized = fields.Bool(allow_none=True)
    uniform_noise = fields.Bool(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ExtractGlimpseConfig(**data)

    @post_dump
    def unmake(self, data):
        return ExtractGlimpseConfig.remove_reduced_attrs(data)


class ExtractGlimpseConfig(BaseLayerConfig):
    """Extracts a glimpse from the input tensor.
    (A mirror to tf.image extract_glimpse)

    Returns a set of windows called glimpses extracted at location `offsets`
    from the input tensor. If the windows only partially overlaps the inputs,
    the non overlapping areas will be filled with random noise.

    The result is a 4-D tensor of shape `[batch_size, glimpse_height,
    glimpse_width, channels]`. The channels and batch dimensions are the
    same as that of the input tensor. The height and width of the output
    windows are specified in the `size` parameter.

    The argument `normalized` and `centered` controls how the windows are built:

        * If the coordinates are normalized but not centered, 0.0 and 1.0
            correspond to the minimum and maximum of each height and width
            dimension.
        * If the coordinates are both normalized and centered, they range from -1.0 to 1.0.
            The coordinates (-1.0, -1.0) correspond to the upper left corner, the lower right
            corner is located at (1.0, 1.0) and the center is at (0, 0).
        * If the coordinates are not normalized they are interpreted as numbers of pixels.

    Args:
        size: A `Tensor` of type `int32`.
            A 1-D tensor of 2 elements containing the size of the glimpses to extract.
            The glimpse height must be specified first, following by the glimpse width.
        offsets: A `Tensor` of type `float32`.
            A 2-D integer tensor of shape `[batch_size, 2]` containing
            the y, x locations of the center of each window.
        centered: An optional `bool`. Defaults to `True`.
            indicates if the offset coordinates are centered relative to the image,
            in which case the (0, 0) offset is relative to the center of the input images.
            If false, the (0,0) offset corresponds to the upper left corner of the input images.
        normalized: An optional `bool`. Defaults to `True`.
            indicates if the offset coordinates are normalized.
        uniform_noise: An optional `bool`. Defaults to `True`.
            indicates if the noise should be generated using a
            uniform distribution or a Gaussian distribution.
        name: A name for the operation (optional).

    Returns:
        A `Tensor` of type `float32`.
        A tensor representing the glimpses `[batch_size, glimpse_height, glimpse_width, channels]`.
    """
    SCHEMA = ExtractGlimpseSchema
    IDENTIFIER = 'ExtractGlimpse'

    def __init__(self,
                 size,
                 offsets,
                 centered=None,
                 normalized=None,
                 uniform_noise=None,
                 **kwargs):
        super(ExtractGlimpseConfig, self).__init__(**kwargs)
        self.size = size
        self.offsets = offsets
        self.centered = centered
        self.normalized = normalized
        self.uniform_noise = uniform_noise


class ToBoundingBoxSchema(BaseLayerSchema):
    offset_height = fields.Int(validate=lambda n: n >= 0)
    offset_width = fields.Int(validate=lambda n: n >= 0)
    target_height = fields.Int(validate=lambda n: n >= 0)
    target_width = fields.Int(validate=lambda n: n >= 0)
    method = fields.Str(allow_none=True, validate=validate.OneOf(['crop', 'pad']))
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ToBoundingBoxConfig(**data)

    @post_dump
    def unmake(self, data):
        return ToBoundingBoxConfig.remove_reduced_attrs(data)


class ToBoundingBoxConfig(BaseLayerConfig):
    """Pad/Crop `image` with zeros to the specified `height` and `width`.
    (A mirror to tf.image pad_to_bounding_box and crop_to_bounding_box)

    If method == 'pad':
        Adds `offset_height` rows of zeros on top, `offset_width` columns of
        zeros on the left, and then pads the image on the bottom and right
        with zeros until it has dimensions `target_height`, `target_width`.

        This op does nothing if `offset_*` is zero and the image already has size
        `target_height` by `target_width`.

    If method == 'crop':
        Crops an image to a specified bounding box.

        This op cuts a rectangular part out of `image`. The top-left corner of the
        returned image is at `offset_height, offset_width` in `image`, and its
        lower-right corner is at `offset_height + target_height, offset_width + target_width`.

    Args:
        offset_height:
            * pad: Number of rows of zeros to add on top.
            * crop: Vertical coordinate of the top-left corner of the result the input.
        offset_width:
            * pad: Number of columns of zeros to add on the left.
            * crop: Horizontal coordinate of the top-left corner of the result in the input.
        target_height: Height of output image.
        target_width: Width of output image.
        method: `crop` or `pad`

    Returns:
        If `image` was 4-D, a 4-D float Tensor of shape
        `[batch, target_height, target_width, channels]`
        If `image` was 3-D, a 3-D float Tensor of shape
        `[target_height, target_width, channels]`

    Raises:
        ValueError: If the shape of `image` is incompatible with the `offset_*` or
        `target_*` arguments, or either `offset_height` or `offset_width` is negative.

    Polyaxonfile usage:

    ```yaml
    - ToBoundingBox:
        offset_height: 10
        offset_width: 10
        target_height: 100
        target_width: 100
    ```
    """
    SCHEMA = ToBoundingBoxSchema
    IDENTIFIER = 'ToBoundingBox'

    def __init__(self,
                 offset_height,
                 offset_width,
                 target_height,
                 target_width,
                 method='crop',
                 **kwargs):
        super(ToBoundingBoxConfig, self).__init__(**kwargs)
        self.offset_height = offset_height
        self.offset_width = offset_width
        self.target_height = target_height
        self.target_width = target_width
        self.method = method


class FlipSchema(BaseLayerSchema):
    axis = fields.Int(allow_none=True, validate=lambda n: n >= 0)
    is_random = fields.Bool(allow_none=True)
    seed = fields.Int(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return FlipConfig(**data)

    @post_dump
    def unmake(self, data):
        return FlipConfig.remove_reduced_attrs(data)


class FlipConfig(BaseLayerConfig):
    """Flip (randomly) an image/images.
    (A mirror to tf.image flip_left_right, flip_up_down, random_flip_left_right, and
    random_flip_up_down)

     if axis is 0:
        * flip horizontally (left to right)
     if axis is 1:
        * vertically (upside down).

    Outputs the contents of `images` flipped along the given axis.

    Args:
        axis: `int`. 0 for horizontal, 1 for vertical
        is_random: `bool`, If True, flip randomly.
        seed: A Python integer. Used to create a random seed. See @{tf.set_random_seed}.

    Returns:
        If `image` was 4-D, a 4-D float Tensor of shape
        `[batch, target_height, target_width, channels]`
        If `image` was 3-D, a 3-D float Tensor of shape
        `[target_height, target_width, channels]

    Raises:
        ValueError: if the shape of `image` not supported.

    Polyaxonfile usage:

    ```yaml
    - FlipSchema:
        axis: 0
    ```
    """
    SCHEMA = FlipSchema
    IDENTIFIER = 'Flip'

    def __init__(self, axis=0, is_random=False, seed=None, **kwargs):
        super(FlipConfig, self).__init__(**kwargs)
        self.axis = axis
        self.is_random = is_random
        self.seed = seed


class TransposeSchema(BaseLayerSchema):
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return TransposeConfig(**data)

    @post_dump
    def unmake(self, data):
        return TransposeConfig.remove_reduced_attrs(data)


class TransposeConfig(BaseLayerConfig):
    """Transpose an image/images by swapping the first and second dimension.
    (A mirror to tf.image transpose_image)

    Returns:
        If `image` was 4-D, a 4-D float Tensor of shape
        `[batch, target_height, target_width, channels]`
        If `image` was 3-D, a 3-D float Tensor of shape
        `[target_height, target_width, channels]

    Raises:
        ValueError: if the shape of `image` not supported.

    Polyaxonfile usage:

    ```yaml
    - Transpose:
    ```
    """
    SCHEMA = TransposeSchema
    IDENTIFIER = 'Transpose'


class Rotate90Schema(BaseLayerSchema):
    k = fields.Int(allow_none=True)
    is_random = fields.Bool(allow_none=True)
    seed = fields.Int(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return Rotate90Config(**data)

    @post_dump
    def unmake(self, data):
        return Rotate90Config.remove_reduced_attrs(data)


class Rotate90Config(BaseLayerConfig):
    """Rotate (randomly) images counter-clockwise by 90 degrees.
    (A mirror to tf.image rot90)

    Args:
        k: A scalar integer. The number of times the image is rotated by 90 degrees.
        is_random: `bool`, If True, adjust randomly.
        seed: A Python integer. Used to create a random seed. See @{tf.set_random_seed}.
        name: A name for this operation (optional).

    Returns:
        If `image` was 4-D, a 4-D float Tensor of shape
        `[batch, target_height, target_width, channels]`
        If `image` was 3-D, a 3-D float Tensor of shape
        `[target_height, target_width, channels]

    Raises:
        ValueError: if the shape of `image` not supported.

    Polyaxonfile usage:

    ```yaml
    - Rotate90:
        k: 2
    ```
    """
    SCHEMA = Rotate90Schema
    IDENTIFIER = 'Rotate90'

    def __init__(self, k=1, is_random=False, seed=None, **kwargs):
        super(Rotate90Config, self).__init__(**kwargs)
        self.k = k
        self.is_random = is_random
        self.seed = seed


class ConvertColorSpaceSchema(BaseLayerSchema):
    from_space = fields.Str(validate=validate.OneOf(['rgb', 'grayscale', 'hsv']))
    to_space = fields.Str(validate=validate.OneOf(['rgb', 'grayscale', 'hsv']))
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ConvertColorSpaceConfig(**data)

    @post_dump
    def unmake(self, data):
        return ConvertColorSpaceConfig.remove_reduced_attrs(data)


class ConvertColorSpaceConfig(BaseLayerConfig):
    """Converts one or more images from RGB to Grayscale.
    (A mirror to tf.image rgb_to_grayscale, rgb_to_hsv, grayscale_to_rgb, and hsv_to_rgb)

    Outputs a tensor of the same `DType` and rank as `images`.

    Possible conversions:
        * rgb_to_grayscale: The size of the last dimension of the output is 1,
            containing the Grayscale value of the pixels.
        * grayscale_to_rgb: The size of the last dimension of the output is 3,
            containing the RGB value of the pixels.
        * hsv_to_rgb: The output is only well defined if the value in `images` are in `[0,1]`.
        * rgb_to_hsv: The output is only well defined if the value in `images` are in `[0,1]`.
            `output[..., 0]` contains hue, `output[..., 1]` contains saturation, and
            `output[..., 2]` contains value. All HSV values are in `[0,1]`. A hue of 0
            corresponds to pure red, hue 1/3 is pure green, and 2/3 is pure blue.
        * grayscale_to_hsv: grayscale_to_rgb then rgb_to_hsv
        * hsv_to_grayscale: hsv_to_rgb then rgb_to_grayscale.

    Args:
        images: 4-D Tensor of shape `[batch, height, width, channels]`
        from_space: The color to convert from.
        to_space: The color space to convert to.
        name: A name for the operation (optional).

    Returns:
        The converted image(s).

    Polyaxonfile usage:

    ```yaml
    - ConvertColorSpace:
        from_space: rgb
        to_space: grayscale
    ```
    """
    SCHEMA = ConvertColorSpaceSchema
    IDENTIFIER = 'ConvertColorSpace'

    def __init__(self, from_space, to_space, **kwargs):
        super(ConvertColorSpaceConfig, self).__init__(**kwargs)
        self.from_space = from_space
        self.to_space = to_space


class ConvertImagesDtypeSchema(BaseLayerSchema):
    dtype = DType()
    saturate = fields.Bool(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return ConvertImagesDtypeConfig(**data)

    @post_dump
    def unmake(self, data):
        return ConvertImagesDtypeConfig.remove_reduced_attrs(data)


class ConvertImagesDtypeConfig(BaseLayerConfig):
    """Convert image(s) to `dtype`, scaling its values if needed.
    (A mirror to tf.image convert_image_dtype)

    Images that are represented using floating point values are expected to have
    values in the range [0,1). Image data stored in integer data types are
    expected to have values in the range `[0,MAX]`, where `MAX` is the largest
    positive representable number for the data type.

    This op converts between data types, scaling the values appropriately before
    casting.

    Note that converting from floating point inputs to integer types may lead to
    over/underflow problems. Set saturate to `True` to avoid such problem in
    problematic conversions. If enabled, saturation will clip the output into the
    allowed range before performing a potentially dangerous cast (and only before
    performing such a cast, i.e., when casting from a floating point to an integer
    type, and when casting from a signed to an unsigned type; `saturate` has no
    effect on casts between floats, or on casts that increase the type's range).

    Args:
        dtype: A `DType` to convert `image` to.
        saturate: If `True`, clip the input before casting (if necessary).
        name: A name for this operation (optional).

    Returns:
        `image`, converted to `dtype`.

    Polyaxonfile usage:

    ```yaml
    - ConvertImagesDtype:
        dtype: float32
    ```
    """
    SCHEMA = ConvertImagesDtypeSchema
    IDENTIFIER = 'ConvertImagesDtype'

    def __init__(self, dtype, saturate=False, **kwargs):
        super(ConvertImagesDtypeConfig, self).__init__(**kwargs)
        self.dtype = dtype
        self.saturate = saturate


class AdjustBrightnessSchema(BaseLayerSchema):
    delta = fields.Float()
    is_random = fields.Bool(allow_none=True)
    seed = fields.Int(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return AdjustBrightnessConfig(**data)

    @post_dump
    def unmake(self, data):
        return AdjustBrightnessConfig.remove_reduced_attrs(data)


class AdjustBrightnessConfig(BaseLayerConfig):
    """Adjust (randomly) the brightness of RGB or Grayscale images.
    (A mirror to tf.image adjust_brightness, random_birightness)

    This is a convenience method that converts an RGB image to float
    representation, adjusts its brightness, and then converts it back to the
    original data type. If several adjustments are chained it is advisable to
    minimize the number of redundant conversions.

    The value `delta` is added to all components of the tensor `image`. Both
    `image` and `delta` are converted to `float` before adding (and `image` is
    scaled appropriately if it is in fixed-point representation). For regular
    images, `delta` should be in the range `[0,1)`, as it is added to the image in
    floating point representation, where pixel values are in the `[0,1)` range.

    If `is_random` is `True`, adjust brightness using a value randomly picked in the
    interval `[-delta, delta)`.

    Args:
        images: A tensor.
        delta: `float`. Amount to add to the pixel values.
        is_random: `bool`, If True, adjust randomly.
        seed: A Python integer. Used to create a random seed. See @{tf.set_random_seed}.

    Returns:
        A brightness-adjusted tensor of the same shape and type as `images`.

    Polyaxonfile usage:

    ```yaml
    - AdjustBrightness:
        delta: 0.5
        is_random: true
    ```
    """
    SCHEMA = AdjustBrightnessSchema
    IDENTIFIER = 'AdjustBrightness'

    def __init__(self, delta, is_random=False, seed=None, **kwargs):
        super(AdjustBrightnessConfig, self).__init__(**kwargs)
        self.delta = delta
        self.is_random = is_random
        self.seed = seed


class AdjustContrastSchema(BaseLayerSchema):
    contrast_factor = fields.Float()
    contrast_factor_max = fields.Float(allow_none=True)
    is_random = fields.Bool(allow_none=True)
    seed = fields.Int(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return AdjustContrastConfig(**data)

    @post_dump
    def unmake(self, data):
        return AdjustContrastConfig.remove_reduced_attrs(data)


class AdjustContrastConfig(BaseLayerConfig):
    """Adjust (randomly) the contrast of RGB or grayscale images by contrast factor.
    (A mirror to tf.image adjust_contrast, random_contrast)

    This is a convenience method that converts an RGB image to float
    representation, adjusts its contrast, and then converts it back to the
    original data type. If several adjustments are chained it is advisable to
    minimize the number of redundant conversions.

    `images` is a tensor of at least 3 dimensions.  The last 3 dimensions are
    interpreted as `[height, width, channels]`.  The other dimensions only
    represent a collection of images, such as `[batch, height, width, channels].`

    Contrast is adjusted independently for each channel of each image.

    For each channel, this Op computes the mean of the image pixels in the
    channel and then adjusts each component `x` of each pixel to
    `(x - mean) * contrast_factor + mean`.

    If `is_random` is `True`: Equivalent to `adjust_contrast()` but the value is
        randomly picked in the interval `[contrast_factor, contrast_factor_max]`.

    Args:
        contrast_factor: `float`.  Lower bound for the random contrast factor.
        contrast_factor_max: `float`.  Upper bound for the random contrast factor.
            Used for random adjustment.
        is_random: `bool`, If True, adjust randomly.
        seed: A Python integer. Used to create a random seed. See @{tf.set_random_seed}.

    Returns:
        The contrast-adjusted tensor.

    Raises:
        ValueError: if `contrast_factor_max <= contrast_factor`
                    if `contrast_factor < 0`
                    if `contrast_factor_max` is None (for random.)

    Polyaxonfile usage:

    ```yaml
    - AdjustContrast:
        contrast_factor: 0.5
        is_random: true
    ```
    """
    SCHEMA = AdjustContrastSchema
    IDENTIFIER = 'AdjustContrast'

    def __init__(self,
                 contrast_factor,
                 contrast_factor_max=None,
                 is_random=False,
                 seed=None,
                 **kwargs):
        super(AdjustContrastConfig, self).__init__(**kwargs)
        self.contrast_factor = contrast_factor
        self.contrast_factor_max = contrast_factor_max
        self.is_random = is_random
        self.seed = seed


class AdjustHueSchema(BaseLayerSchema):
    delta = fields.Float(validate=validate.Range(min=-1., max=1.))
    is_random = fields.Bool(allow_none=True)
    seed = fields.Int(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return AdjustHueConfig(**data)

    @post_dump
    def unmake(self, data):
        return AdjustHueConfig.remove_reduced_attrs(data)


class AdjustHueConfig(BaseLayerConfig):
    """Adjust (randomly) hue of an RGB images.
    (A mirror to tf.image adjust_hue, random_hue)

    This is a convenience method that converts an RGB image to float
    representation, converts it to HSV, add an offset to the hue channel, converts
    back to RGB and then back to the original data type. If several adjustments
    are chained it is advisable to minimize the number of redundant conversions.

    `image` is an RGB image.  The image hue is adjusted by converting the
    image to HSV and rotating the hue channel (H) by `delta`.
    The image is then converted back to RGB.

    `delta` must be in the interval `[-1, 1]`.

    If `is_random` is `True` adjust hue but uses a value randomly picked in
    the interval `[-delta, delta]`.

    Args:
        delta: float.  How much to add to the hue channel.
        is_random: `bool`, If True, adjust randomly.
        seed: A Python integer. Used to create a random seed. See @{tf.set_random_seed}.
        name: A name for this operation (optional).

    Returns:
        Adjusted image(s), same shape and DType as `image`.

    Polyaxonfile usage:

    ```yaml
    - AdjustHue:
        delta: 0.5
        is_random: true
    ```
    """
    SCHEMA = AdjustHueSchema
    IDENTIFIER = 'AdjustHue'

    def __init__(self, delta, is_random=False, seed=None, **kwargs):
        super(AdjustHueConfig, self).__init__(**kwargs)
        self.delta = delta
        self.is_random = is_random
        self.seed = seed


class AdjustSaturationSchema(BaseLayerSchema):
    saturation_factor = fields.Float()
    saturation_factor_max = fields.Float(allow_none=True)
    is_random = fields.Bool(allow_none=True)
    seed = fields.Int(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return AdjustSaturationConfig(**data)

    @post_dump
    def unmake(self, data):
        return AdjustSaturationConfig.remove_reduced_attrs(data)


class AdjustSaturationConfig(BaseLayerConfig):
    """Adjust (randomly) saturation of RGB images.
    (A mirror to tf.image adjust_saturation, random_saturation)

    This is a convenience method that converts an RGB image to float
    representation, converts it to HSV, add an offset to the saturation channel,
    converts back to RGB and then back to the original data type. If several
    adjustments are chained it is advisable to minimize the number of redundant
    conversions.

    The image saturation is adjusted by converting the images to HSV and
    multiplying the saturation (S) channel by `saturation_factor` and clipping.
    The images is then converted back to RGB.

    If `is_random` is `True` adjust saturation but uses a value randomly picked in
    the interval `[saturation_factor, saturation_factor_max]`.

    Args:
        saturation_factor: float.  Lower bound for the random saturation factor.
        saturation_factor_max: float.  Upper bound for the random saturation factor.
        is_random: `bool`, If True, adjust randomly.
        seed: An operation-specific seed. It will be used in conjunction
          with the graph-level seed to determine the real seeds that will be
          used in this operation. Please see the documentation of
          set_random_seed for its interaction with the graph-level random seed.
        name: A name for this operation (optional).

    Returns:
        Adjusted image(s), same shape and DType as `image`.

    Raises:

        ValueError: if `saturation_factor_max <= saturation_factor`
                    if `saturation_factor < 0`
                    if `saturation_factor_max is None (for random.)`

    Polyaxonfile usage:

    ```yaml
    - AdjustSaturation:
        saturation_factor: 0.5
        saturation_factor_max: 1.
    ```
    """
    SCHEMA = AdjustSaturationSchema
    IDENTIFIER = 'AdjustSaturation'

    def __init__(self, saturation_factor, saturation_factor_max=None, is_random=False,
                 seed=None, **kwargs):
        super(AdjustSaturationConfig, self).__init__(**kwargs)
        self.saturation_factor = saturation_factor
        self.saturation_factor_max = saturation_factor_max
        self.is_random = is_random
        self.seed = seed


class AdjustGammaSchema(BaseLayerSchema):
    gamma = fields.Float(validate=lambda n: n > 0.)
    gain = fields.Float(allow_none=True)
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return AdjustGammaConfig(**data)

    @post_dump
    def unmake(self, data):
        return AdjustGammaConfig.remove_reduced_attrs(data)


class AdjustGammaConfig(BaseLayerConfig):
    """Performs Gamma Correction on the input image.
    Also known as Power Law Transform. This function transforms the
    input image pixelwise according to the equation Out = In**gamma
    after scaling each pixel to the range 0 to 1.
    (A mirror to tf.image adjust_gamma)

    Args:
        gamma : A scalar. Non negative real number.
        gain  : A scalar. The constant multiplier.

    Returns:
        A Tensor. Gamma corrected output image.

    Notes:
        For gamma greater than 1, the histogram will shift towards left and
        the output image will be darker than the input image.
        For gamma less than 1, the histogram will shift towards right and
        the output image will be brighter than the input image.

    References:
        [1] http://en.wikipedia.org/wiki/Gamma_correction

    Polyaxonfile usage:

    ```yaml
    - AdjustGamma:
        gamma: 1
    ```
    """
    SCHEMA = AdjustGammaSchema
    IDENTIFIER = 'AdjustGamma'

    def __init__(self, gamma=1, gain=1, **kwargs):
        super(AdjustGammaConfig, self).__init__(**kwargs)
        self.gamma = gamma
        self.gain = gain


class StandardizationSchema(BaseLayerSchema):
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return StandardizationConfig(**data)

    @post_dump
    def unmake(self, data):
        return StandardizationConfig.remove_reduced_attrs(data)


class StandardizationConfig(BaseLayerConfig):
    """Linearly scales `image` to have zero mean and unit norm.
    (A mirror to tf.image per_image_standardization)

    This op computes `(x - mean) / adjusted_stddev`, where `mean` is the average
    of all values in image, and
    `adjusted_stddev = max(stddev, 1.0/sqrt(image.NumElements()))`.

    `stddev` is the standard deviation of all values in `image`. It is capped
    away from zero to protect against division by 0 when handling uniform images.

    Returns:
        The standardized image with same shape as `image`.

    Raises:
        ValueError: if the shape of 'image' is incompatible with this function.

    Polyaxonfile usage:

    ```yaml
    - Standardization:
    ```
    """
    SCHEMA = StandardizationSchema
    IDENTIFIER = 'Standardization'


class DrawBoundingBoxesSchema(BaseLayerSchema):
    boxes = fields.List(fields.Float(), validate=validate.Length(equal=3))
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return DrawBoundingBoxesConfig(**data)

    @post_dump
    def unmake(self, data):
        return DrawBoundingBoxesConfig.remove_reduced_attrs(data)


class DrawBoundingBoxesConfig(BaseLayerConfig):
    """Draw bounding boxes on a batch of images.
    (A mirror to tf.image draw_bounding_boxes)

    Outputs a copy of `images` but draws on top of the pixels zero or more bounding
    boxes specified by the locations in `boxes`. The coordinates of the each
    bounding box in `boxes` are encoded as `[y_min, x_min, y_max, x_max]`. The
    bounding box coordinates are floats in `[0.0, 1.0]` relative to the width and
    height of the underlying image.

    For example, if an image is 100 x 200 pixels and the bounding box is
    `[0.1, 0.2, 0.5, 0.9]`, the bottom-left and upper-right coordinates of the
    bounding box will be `(10, 40)` to `(50, 180)`.

    Parts of the bounding box may fall outside the image.

    Args:
        boxes: A `Tensor` of type `float32`.
            3-D with shape `[batch, num_bounding_boxes, 4]` containing bounding boxes.
        name: A name for the operation (optional).

    Returns:
        A `Tensor`. Has the same type as `images`.
        4-D with the same shape as `images`. The batch of input images with
        bounding boxes drawn on the images.

    Polyaxonfile usage:

    ```yaml
    - DrawBoundingBoxes:
        boxes: [0.1, 0.2, 0.5, 0.9]
    ```
    """
    SCHEMA = DrawBoundingBoxesSchema
    IDENTIFIER = 'DrawBoundingBoxes'

    def __init__(self, boxes, **kwargs):
        super(DrawBoundingBoxesConfig, self).__init__(**kwargs)
        self.boxes = boxes


class TotalVariationSchema(BaseLayerSchema):
    name = fields.Str(allow_none=True)

    class Meta:
        ordered = True

    @post_load
    def make(self, data):
        return TotalVariationConfig(**data)

    @post_dump
    def unmake(self, data):
        return TotalVariationConfig.remove_reduced_attrs(data)


class TotalVariationConfig(BaseLayerConfig):
    """Calculate and return the total variation for one or more images.

    (A mirror to tf.image total_variation)

    The total variation is the sum of the absolute differences for neighboring
    pixel-values in the input images. This measures how much noise is in the
    images.

    This can be used as a loss-function during optimization so as to suppress
    noise in images. If you have a batch of images, then you should calculate
    the scalar loss-value as the sum:
    `loss = tf.reduce_sum(tf.image.total_variation(images))`

    This implements the anisotropic 2-D version of the formula described here:

    https://en.wikipedia.org/wiki/Total_variation_denoising

    Args:
        name: A name for the operation (optional).

    Raises:
        ValueError: if images.shape is not a 3-D or 4-D vector.

    Returns:
        The total variation of `images`.

        If `images` was 4-D, return a 1-D float Tensor of shape `[batch]` with the
        total variation for each image in the batch.
        If `images` was 3-D, return a scalar float with the total variation for that image.

    Polyaxonfile usage:

    ```yaml
    - TotalVariation:
    ```
    """
    SCHEMA = TotalVariationSchema
    IDENTIFIER = 'TotalVariation'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from polyaxon_schemas.polyaxonfile import reader


class TestREader(TestCase):
    def test_reads_yaml_files(self):
        config = reader.read('tests/fixtures/simple_file.yml')
        assert config is not None

    def test_reads_json_files(self):
        config = reader.read('tests/fixtures/simple_json_file.json')
        assert config == {'x': 'y'}

    def test_reads_confi_map(self):
        config = reader.read([{'x': 'y'}, {1: 2}, {'x': 'override y'}])
        assert config == {'x': 'override y', 1: 2}

    def test_reads_yaml_stream(self):
        stream = """---
        x: y
        1: 2
        """
        config = reader.read(stream)
        assert config == {'x': 'y', 1: 2}

    def test_reads_json_stream(self):
        stream = """---
        {x: y, 1: 2}
        """
        config = reader.read(stream)
        assert config is not None
